;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                            ;;;
;;;  Model of CABN pyramids task, only regular problems        ;;;
;;;                                                            ;;;
;;;  This model is part of:                                    ;;;
;;;  Borst & Anderson (submitted). A Step-By-Step Tutorial     ;;;
;;;  on using the Cognitive Architecture ACT-R in combination  ;;; 
;;;  with fMRI Data. Journal of Mathematical Psychology.       ;;;
;;;                                                            ;;;
;;;  Jelmer Borst                                              ;;;
;;;  University of Groningen                                   ;;;
;;;  j.p.borst@rug.nl                                          ;;;
;;;                                                            ;;;
;;;  150731                                                  ;;;
;;;                                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EXPERIMENT ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; parameters

(defparameter *position* 150) ;position of response cursor in window


;n-back = repetition detection task
(defparameter *in-nback* nil)
(defparameter *real-clicks* nil)
(defparameter *nback-match* 0)
(defparameter *rd-letter* nil)

;event for timeout if no response
(defparameter *timeout-event* nil)

;events
(defparameter *experiment-events* nil)
(defparameter *problem-events* nil)

;bookkeeping with globals
(defparameter *subject-id* nil)
(defparameter *blocknum* nil)
(defparameter *problemnum* nil)
(defparameter *problem* nil)
(defparameter *problem-result* nil) ;current result
(defparameter *problem-start-time* nil)
(defparameter *results* nil) ;all results


;holds all problems
(defparameter *problems* nil)

(defparameter *test-problems*
  '(
    (1 (
        (
         ( 4 $ 3 = 9 Small Value )
         ( 6 $ 4 = 18 Large Value )
         ( 7 $ 3 = 18 Small Height )
         ( 8 $ 5 = 30 Large Base )
         )))))

(setf *problems* *test-problems*) ;will be replaced with real problems when simulating real subjects


; depending on problem type (solve for value, height, or base), set right variable to 'x'
(defun transform (problem)
  (let* ((key (car (last problem)))
         (first (if (eq key 'base) 'x (first problem)))
         (second (if (eq key 'height) 'x (third problem)))
         (third (if (eq key 'value) 'x (fifth problem))))
    (list first second third)))

;;run single trial
(defun run-4-trials ()
  (setf *problems* *test-problems*) 
  (let ((nsub (length *problems*)))
    (dotimes (subj nsub)
      (setf *results* nil)
      (let ((nblock (length (second (nth subj *problems*)))))
        (dotimes (bl nblock)
          (let ((ntrial (length (nth bl (second (nth subj *problems*))))))
            (dotimes (tr ntrial)
              (run-trial-human (+ subj 1) (+ bl 1) (+ tr 1)))))))))
      
(defun run-trial-human (subject-index blocknum problemnum)
  (reset)

  (let* ((window (open-exp-window "Pyramid Experiment" :visible t))
        (subject-id (first (nth subject-index *problems*)))
        (problem (nth (- problemnum 1) (nth (- blocknum 1) (second (nth (- subject-index 1) *problems*)))))
        (items (transform problem))
        (wait-time 4))

        (setf *subject-id* subject-id) ; globals, essentially passed to schedule-post-answer
        (setf *blocknum* blocknum)
        (setf *problemnum* problemnum)
        (setf *problem* problem)
        (setf *problem-result* nil)
        
        (setf *position* 150) ;response cursor in window
        (install-device window)
        (setf *real-clicks* t) ;prevents clicking buttons for safety (could probably be removed)
        
        ;schedule start trial in 4 seconds from now (fixations is 4 seconds, model assumes nothing happens during fixation)
        (schedule-event-relative wait-time
                                 (lambda () 
                                   (princ (format nil "Presenting ~A $ ~A = ~A~%" (first items) (second items) (third items)))
                                   
                                   (setf *problem-start-time* (/ (get-time) 1000.0)) ;bookkeeping
                                   (mod-focus-fct (list 'state 'encode-base 'step 'ready 'next-state nil 'role nil 'value nil 'current-term nil 
                                                        'current-problem 1 'current-problem-position 'P1)) ;set goal
                                  
                                   (present-problem items) ;present problem
                                   (proc-display :clear t))) ;tell model that interface updated

        (setf *timeout-event* (schedule-event-relative (+ wait-time 30) 'schedule-post-answer)) ;in case it times out this is scheduled already (30 seconds after start)
        (run 1000 :real-time t))) ;run for 1000 seconds or until model doesn't have anything left to do
  
    

;;run single trial
(defun run-trial (subject-index blocknum problemnum)

  (reset)

  (let* ((window (open-exp-window "Pyramid Experiment" :visible nil))
        (subject-id (first (nth (- subject-index 1) *problems*)))
        (problem (nth (- problemnum 1) (nth (- blocknum 1) (second (nth (- subject-index 1) *problems*)))))
        (items (transform problem))
        (wait-time 4))

        (setf *subject-id* subject-id) ; globals, essentially passed to schedule-post-answer
        (setf *blocknum* blocknum)
        (setf *problemnum* problemnum)
        (setf *problem* problem)
        (setf *problem-result* nil)
        
        (setf *position* 150) ;response cursor in window
        (install-device window)
        (setf *real-clicks* nil) ;prevents clicking buttons for safety (could probably be removed)
        
        ;schedule start trial in 4 seconds from now (fixations is 4 seconds, model assumes nothing happens during fixation)
        (schedule-event-relative wait-time
                                 (lambda () 
                                   (princ (format nil "Presenting ~A $ ~A = ~A~%" (first items) (second items) (third items)))
                                   
                                   (setf *problem-start-time* (/ (get-time) 1000.0)) ;bookkeeping
                                   (mod-focus-fct (list 'state 'encode-base 'step 'ready 'next-state nil 'role nil 'value nil 'current-term nil 
                                                        'current-problem 1 'current-problem-position 'P1)) ;set goal
                                   
                                   (present-problem items) ;present problem
                                   (proc-display :clear t))) ;tell model that interface updated

        (setf *timeout-event* (schedule-event-relative (+ wait-time 30) 'schedule-post-answer)) ;in case it times out this is scheduled already (30 seconds after start)
        (run 1000 :real-time nil)) ;run for 1000 seconds or until model doesn't have anything left to do
  
  (if *save-mod-preds* ;if we want to calculate MRI predictions, we save module activity per trial.
      (write-fmri-results (ensure-directories-exist
                           (make-pathname :name (format nil "model_act_~a_~a_~a" *subject-id* *blocknum* *problemnum*)  :type "txt"  
                           :defaults *output-pathname-modact*)))))
    

;; present problem in window
(defun present-problem (items)

  ;keyboard
  (add-button-to-exp-window :x 80 :y 110 :height 30 :width 120 :text "Submit" :action #'(lambda (button) (declare (ignore button)) (handle-button "submit")))
  (add-button-to-exp-window :x 80 :y 140 :height 30 :width 40 :text "7" :action #'(lambda (button) (declare (ignore button)) (handle-button "7")))
  (add-button-to-exp-window :x 80 :y 170 :height 30 :width 40 :text "4" :action #'(lambda (button) (declare (ignore button)) (handle-button "4")))
  (add-button-to-exp-window :x 80 :y 200 :height 30 :width 40 :text "1" :action #'(lambda (button) (declare (ignore button)) (handle-button "1")))
  (add-button-to-exp-window :x 80 :y 230 :height 30 :width 40 :text "/" :action #'(lambda (button) (declare (ignore button)) (handle-button "/")))
  (add-button-to-exp-window :x 80 :y 260 :height 30 :width 120 :text "Space" :action #'(lambda (button) (declare (ignore button)) (handle-button "space")))
  (add-button-to-exp-window :x 120 :y 140 :height 30 :width 40 :text "8" :action #'(lambda (button) (declare (ignore button)) (handle-button "8")))
  (add-button-to-exp-window :x 120 :y 170 :height 30 :width 40 :text "5" :action #'(lambda (button) (declare (ignore button)) (handle-button "5")))
  (add-button-to-exp-window :x 120 :y 200 :height 30 :width 40 :text "2" :action #'(lambda (button) (declare (ignore button)) (handle-button "2")))
  (add-button-to-exp-window :x 120 :y 230 :height 30 :width 40 :text "0" :action #'(lambda (button) (declare (ignore button)) (handle-button "0")))
  (add-button-to-exp-window :x 160 :y 140 :height 30 :width 40 :text "9" :action #'(lambda (button) (declare (ignore button)) (handle-button "9")))
  (add-button-to-exp-window :x 160 :y 170 :height 30 :width 40 :text "6" :action #'(lambda (button) (declare (ignore button)) (handle-button "6")))
  (add-button-to-exp-window :x 160 :y 200 :height 30 :width 40 :text "3" :action #'(lambda (button) (declare (ignore button)) (handle-button "3")))
  (add-button-to-exp-window :x 160 :y 230 :height 30 :width 40 :text "-" :action #'(lambda (button) (declare (ignore button)) (handle-button "-")))

  (cond ((atom (first items))
         (add-text-to-exp-window :text (format nil "~A" (first items)) :x 75 :y 60 :width 25))
        (t (add-text-to-exp-window :text (format nil "~A" (first (first items))) :x 50 :y 60 :width 25)
           (add-text-to-exp-window :text (format nil "~A" (second (first items))) :x 75 :y 60 :width 25)))
  (add-text-to-exp-window :text "$" :x 100 :y 60 :width 25)
  (add-text-to-exp-window :text (format nil "~A" (second items)) :x 125 :y 60 :width 25)
  (add-text-to-exp-window :text "=" :x 150 :y 60 :width 25)
  (cond ((atom (third items)) 
         (add-text-to-exp-window :text (format nil "~A" (third items)) :x 175 :y 60 :width 25))
        (t (add-text-to-exp-window :text (concatenate 'string (format nil "~A" (first (third items))) "_" 
                                                      (format nil "~A"(second (third items)))) :x 175 :y 60 :width 50))))

;function gets called when model clicks button
(defun handle-button (button)
  (if (and *real-clicks* (not *in-nback*))
    (progn 
      (push (list (- (/ (get-time) 1000.0) *problem-start-time*) button) *problem-result*) ; safe RT and responses
      (if (not (string-equal button "submit"))
        (progn 
          (add-text-to-exp-window :text (string button) :x *position* :y 90 :width 25)
          (proc-display) 
          (setf *position* (+ *position* 8)))
        (progn
           ;delete time out when clicking submit, move on to feedback
          (delete-event *timeout-event*)
          (schedule-post-answer)
)))))

;handle feedback
(defun schedule-post-answer () ;removed NB
  (schedule-event-relative
   0 
   (lambda () ; feedback start now
     (setf *problem-result* (reverse *problem-result*)) ;reverse the order of results - these are the keypresses
     (let ((result (augment-key-times *problem-result* *subject-id* *blocknum* *problemnum* *problem*)))
       (push result *problem-result*) ;put the fixed result in
       (clear-exp-window)
       (present-feedback *problem* (fifth result) (seventh result)))
     (proc-display :clear t)))
  (schedule-event-relative ; end feedback
   5
   (lambda () 
     (push *problem-result* *results*)))
  (schedule-break-relative 5.001) ; stop ACT-R running
  nil)


(defun augment-key-times (key-times subject-id blocknum problemnum problem)
  ; parse key-times to an actual number
  ; get the last keystroke time
  ; check against the problem definition to see if the answer is right
  ; return a list with that info in it
  (let (answer last-keystroke is-correct (key (car (last problem))))
    (setf answer
          (cond 
           ((eq (list-length key-times) 2)
            (read-from-string (string (second (first key-times)))))
           ((eq (list-length key-times) 3)
          (+ (read-from-string (string (second (second key-times))))
             (* 10 (read-from-string (string (second (first key-times)))))))
           (t nil)))
    (setf last-keystroke (first (first (last key-times))))
    (setf is-correct 
          (cond
           ((eq key 'base)
            (eq answer (first problem)))
           ((eq key 'height)
            (if (and (eq (first problem) 4) (eq (fifth problem) 10)) ; special case: in 4$X=10, X can be 4 or 5 
                (or (eq answer 4) (eq answer 5))
              (eq answer (third problem))))
           ((eq key 'value)
            (eq answer (fifth problem)))
           (t "error")))
    (list subject-id blocknum problemnum problem answer last-keystroke is-correct)
    ))

(defun present-feedback (problem result iscorrect)
  (let ((base (first problem))
        (height (third problem))
        (value (fifth problem))
        (expansion nil))
    (setf expansion (format nil "~A $ ~A = ~A" base height base))
    (loop for i from 2 to height do
          (setf expansion (concatenate 'string 
                                       expansion (format nil " + ~A"
                                                         (- base (- i 1))))))
    (setf expansion (concatenate 'string expansion (format nil " = ~A" value)))
    (if iscorrect
        (add-text-to-exp-window :text expansion :x 10 :y 60 :color 'green)
      (add-text-to-exp-window :text expansion :x 10 :y 60 :color 'red))
    (add-text-to-exp-window :text (format nil "~A" result) :x 10 :y 90 :color 'blue)
    ))


;saving stuff to disk
(defparameter *save-mod-preds* nil) ;save model predictions in form of module activity
(defparameter *output-pathname-modact* ;pathname for module activity
  (merge-pathnames 
   (make-pathname :name "pp-test" :directory '(:relative "model_data_mri") :type "txt")  
    (translate-logical-pathname "ACT-R:"))) ;use current CCL path to start with, put it in relative directory 'model_data'

(defparameter *output-pathname* ;pathname for model behavior
 (merge-pathnames 
   (make-pathname :name "pp-test" :directory '(:relative "model_data") :type "txt")  
      (translate-logical-pathname "ACT-R:")))


(defun save-results (filename)
  (let ((fstream (open filename :direction :output :if-exists :supersede)))
    (mapcar #'(lambda(resultset) 
                  (let* ((data (first resultset))
                         (problem (fourth data))
                         (first-key (second resultset)))
                  ; subjnum blocknum probnum problem type unknown time firstentrytime iscorrect
                  (format fstream "~A ~A ~A ~A~A~A~A~A ~A ~A ~F ~F ~A~%" 
                          (first data) (second data) (third data)
                          (first problem) (second problem) (third problem) ;B $ H
                          (fourth problem) (fifth problem) ; = V
                          (sixth problem) (seventh problem) ; type and unknown
                          (sixth data) ; overall time
                          (first first-key) ; time of first key
                          (seventh data)))) ; is correct
            *results*)
    (close fstream)))


;; write module activity
(defun write-to-file-pred (name lst)
  (with-open-file (out name :direction :output :if-does-not-exist :create :if-exists :supersede)
    (dolist (line lst)
      (dolist (times (cdr line)) 
        (format out "~a ~3$ ~3$~%" (car line) (first times) (cdr times))))))

(defun write-fmri-results (filename)
  (let ((act-trace (parse-trace-lists-for-bold (get-module bold))))
    (write-to-file-pred filename act-trace)))


;; run multiple trials
(defun run-all-problems () ;... that are stored in *problems*
  (let ((nsub (length *problems*)))
    (dotimes (subj nsub)
      (setf *results* nil)
      (let ((nblock (length (second (nth subj *problems*)))))
        (dotimes (bl nblock)
          (let ((ntrial (length (nth bl (second (nth subj *problems*))))))
            (dotimes (tr ntrial)
              (run-trial (+ subj 1) (+ bl 1) (+ tr 1))))))
      ;write results
      (setf *results* (reverse *results*))
      (save-results  (make-pathname :name 
                                    (format nil "model_~a" *subject-id*) 
                                    :defaults *output-pathname*)))))

(defparameter *real-problems* nil)
(defun run-real-problems () ;read in real problems for real subjects, and run them all
 ; (with-open-file (in (make-pathname :name "real_problems" :defaults *output-pathname*))
 ;     (progn (eval (read in)) t))
  (setf *problems* *real-problems*)
  (setf *save-mod-preds* t)
  (run-all-problems))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; MODEL LISP FUNCTIONS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;create number chunks in ACT-R's Declarative Memory
(defparameter *min-dm-number* -20)
(defparameter *max-dm-number* 80)
(defun add-numbers-to-dm ()
  (loop for i from *min-dm-number* to *max-dm-number* do
        (let ((string-val (format nil "~A" i)))
          (add-dm-fct (list (list (intern (concatenate 'string "N" string-val)) 'isa 'equation-token 'string string-val))))))

;set similarities between numbers
(defun set-number-similarities ()
  ; assuming similarities range -1-0
  ; Based on Lebiere (p.48), the similarities are equal to the ratio of the
  ; smaller number to the larger (minus 1 to make them -1 to 0)
  ;
  ; negative numbers don't currently have similarities (all max-difference)
  ; zero is also maximally different from everything else
  (loop for i from 1 to *max-dm-number* do
        (loop for j from (+ i 1) to *max-dm-number* do
              (set-similarities-fct (list (list (number-to-chunk i) (number-to-chunk j)
                                                (- (/ i j) 1)))))))

;this is not relevant in the current model, but necessay to run it
(defun initialize-position-chunks (factor) 
  ; add some chunks to dm that can be used to index the position in a
  ; problem-solving process
  ;
  ; make them similar based the factor:
  ; similarity = factor*(ratio of smaller step # to larger) - 1
  ;
  ; assuming ms/md of 0/-1, factors should range from 0 (everything
  ; maximally different) to 2 (steps 1 and 2 maximally similar, 1 and 3 at
  ; -1/3, ..)
  ;
  ; going above 2 will cause steps 1 and 2 to have a similarity over 0, so its
  ; probably best to avoid that
  (loop for i from 1 to 40 do
        (let ((string-val (format nil "~A" i)))
          (add-dm-fct (list (list (intern (concatenate 'string "P" string-val)) 'isa 'problem-position)))))
  (loop for i from 1 to 40 do
        (loop for j from (+ i 1) to 40 do
              (set-similarities-fct (list (list (intern (concatenate 'string "P" (format nil "~A" i)))
                                                (intern (concatenate 'string "P" (format nil "~A" j)))
                                                (- (* factor (/ i j)) 1)))))))

(defun next-problem-position (current)
  (intern (concatenate 'string "P" (format nil "~A" (+ 1 (chunk-to-number current))))))



; works on number chunks and problem-positions
(defun chunk-to-number (x)
  (read-from-string (subseq (symbol-name x) 1)))
  
; only works on number chunks
(defun number-to-chunk (x)
  (intern (concatenate 'string "N" (format nil "~A" x))))
  
(defun chunk-to-string (x)
  (format nil "~A" (chunk-to-number x)))

(defun string-to-chunk (x)
  (number-to-chunk (read-from-string x)))


;;add addition facts to DM  
(defun generate-addition-fact (addend1 addend2) 
  (list 'isa 'addition-fact 'addend1 (number-to-chunk addend1) 'addend2 (number-to-chunk addend2) 'sum (number-to-chunk (+ addend1 addend2))))

(defun generate-addition-facts (digits1 digits2) 
  (mapcan #'(lambda (x) (mapcar #'(lambda (y) (generate-addition-fact x y)) digits2)) digits1)
)


; adds a set of addition facts to DM, where addend1 takes on each of the values in
; addend1s and addend2 takes on those in addend2s, and assign the new facts the
; given base-level and creation-time
(defun add-addition-facts (addend1s addend2s base-level creation-time)
  (set-base-levels-fct 
   (mapcar #'(lambda(x) (list x base-level creation-time)) 
           (add-dm-fct (generate-addition-facts addend1s addend2s)))))


(defun add-weighted-addition-facts (factor)
  (let ((digits '(-10 -9 -8 -7 -6 -5 -4 -3 -2 -1 1 2 3 4 5 6 7 8 9 10 ))
        (fac1 2) ;2
        (fac2 9)) ;9

    (mapcan #'(lambda (x) 
                (mapcar #'(lambda (y) (set-base-levels-fct
                                        (mapcar #'(lambda(chunkname) 
                                                    (list chunkname
                                                          (* factor (* 2222 (* (- fac1 (/ (sqrt (abs x)) fac2)) (- fac1 (/ (sqrt (abs y)) fac2))))) 
                                                          -432000000))
                                                (add-dm-fct (list (generate-addition-fact x y))))))
                            digits)) digits)))


(defun ones (x) (number-to-chunk (mod (chunk-to-number x) 10)))
(defun tens (x) (number-to-chunk (floor (/ (chunk-to-number x) 10))))
(defun combine (x y) (number-to-chunk (+ (* (chunk-to-number x) 10) (chunk-to-number y))))
(defun invert (x) (number-to-chunk (* (chunk-to-number x) -1)))
(defun increment (x) (number-to-chunk (+ (chunk-to-number x) 1)))
(defun decrement (x) (number-to-chunk (+ (chunk-to-number x) -1)))
(defun add (x y) (number-to-chunk (+ (chunk-to-number x) (chunk-to-number y))))
(defun tostring (x) (number-to-chunk x))
(defun issimpleaddition (x y) (let ((num1 (chunk-to-number x))
                             (num2 (chunk-to-number y)))
                      (or (= num1 1)
                          (= num1 0)
                          (= num1 -1)
                          (= num2 1)
                          (= num2 0)
                          (= num2 -1))))


(defparameter *guess-noise* .8) ; really 1-noise

; always-correct guesser- but there's noise in the input!
(defun guess (height value noise)
  (let ((v (chunk-to-number value))
        (h (chunk-to-number height)))
    (number-to-chunk 
      (noisy-int noise 
      (case v
        (10 4)
        (9 (case h (3 4) (2 5)))
        (7 4)
        (15 (case h (5 5) (3 6) (2 8)))
        (14 5)
        (12 5)
        (20 6)
        (18 (case h (4 6) (3 7)))
        (11 6)
        (25 7)
        (22 7)
        (13 7)
        (30 (case h (5 8) (4 9) (3 11)))
        (26 8)
        (21 (case h (3 8) (2 11)))
        (35 9)
        (24 9)
        (17 9)
        (23 12)
        (19 10)
        (27 10)
        (33 12)
        (34 10)
        (40 10))))))

; return starting with probability (prob), and starting +/- 1 with
; probability (1-prob/2) each
(defun noisy-int (prob starting) 
  (if (<  prob (act-r-random 1.0))
    (if (> 0.5 (act-r-random 1.0))
      (+ 1 starting)
      (- starting 1))
    starting))

(defun print-set (var val) 
  (princ (format nil "Setting ~A to ~A.~%" var val)))


;this assumes that the model/subjects know where the buttons are after the training; that they
;do not have to iterate looking over all buttons to find one
(defun find-location-x (s)
  "Hack to find a button with string s"  
  (let ((answer nil)
        (l (visicon-chunks (get-module :vision))))
    (while (and (null answer) l)
        (if (and
             (equal (chunk-slot-value-fct (first l) 'kind) 'TEXT)
             (> (chunk-slot-value-fct (first l) 'screen-y) 120)
             (equalp (string-upcase s)(string-upcase (chunk-real-visual-value (first l)))))
            (setf answer (first l))
          (setf l (Rest l))))
      (if answer (chunk-slot-value-fct answer 'screen-x)
        (progn (format t "~%*** Warning: find-location-x called with ~S which is not on the screen ***~%" s)
          nil))))
  
(defun find-location-y (s)
  "Hack to find a button with string s"   
  (let ((answer nil)
        (l (visicon-chunks (get-module :vision))))
    (while (and (null answer) l)
        (if (and
             (equal (chunk-slot-value-fct (first l) 'kind) 'TEXT)
             (> (chunk-slot-value-fct (first l) 'screen-y) 120)
             (equalp (string-upcase s)(string-upcase (chunk-real-visual-value (first l)))))
            (setf answer (first l))
          (setf l (Rest l))))
      (if answer (chunk-slot-value-fct answer 'screen-y)
        (progn (format t "~%*** Warning: find-location-y called with ~S which is not on the screen ***~%" s)
          nil))))







;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; MODEL ACT-R ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(clear-all)

(define-model pyramids

  (sgp :esc t :bll .5
       :rt -5
       :ol 4
       :ans .4
       :er t)  
 
  (sgp :save-buffer-trace t :traced-buffers (imaginal visual retrieval manual))
  
  (sgp :lf .5
       :v nil
       :trace-detail low
       :show-focus nil
       :trace-mouse nil
       :dcnn nil
       :ncnar nil
       :unstuff-visual-location nil)
  


  (set-visloc-default isa visual-location screen-x lowest screen-y lowest :attended new)
  (start-hand-at-mouse)

  (chunk-type add-pair one1 ten1 one2 ten2 one-ans ten-ans combined-sum carry scratch scratch2 scratch3)
  (chunk-type operator pre action arg1 arg2 next-state alt)
  (chunk-type pyramid base height value current-term unknown-field current-problem current-problem-position scratch)
  
  (chunk-type task state step next-state role  value current-term 
              current-problem current-problem-position guess) 
  
  (chunk-type addition-fact addend1 addend2 sum)
  (chunk-type equation-token string)
  (chunk-type problem-position dummy)
  (chunk-type item-location item location)
  (chunk-type respond value)
  (chunk-type feedback acc)

  (chunk-type fake dummy2)

  (add-numbers-to-dm)
  (set-number-similarities)
  (initialize-position-chunks 0)
  (add-dm 

    (goal isa task state nil step nil next-state nil role nil value nil current-problem 0 current-problem-position P1)
    (x isa equation-token string "x" )
    (dollar-sign isa equation-token string "$")
    (equal-sign isa equation-token string "=")
    (plus-sign isa equation-token string "+")
    (minus-sign isa equation-token string "-")

    (base isa chunk)
    (height isa chunk)
    (value isa chunk)
    (submit isa chunk)
    (space isa chunk)
    (slash isa chunk)
    (minus isa chunk)
    (encode-base isa chunk)
    (match isa chunk)
    (ready isa chunk)
    (encode-height isa chunk)
    (encode-value isa chunk)
    (looking isa chunk)
    (attending isa chunk)
    (begin-solving isa chunk)
    (countdown isa chunk)
    (add-up isa chunk)
    (add-done isa chunk)
    (uv-in-place isa chunk)
    (uh-simple isa chunk)
    (start isa chunk)
    (answer isa chunk)
    (respond isa chunk)
    (prepare-click isa chunk)
    (click isa chunk)
    (attend-entered isa chunk)
    (prepare-click-s isa chunk)
    (click-s isa chunk)
    (done isa chunk)
    (find-feedback isa chunk)
    (attend-feedback isa chunk)
    (findloc isa chunk)
    (attend isa chunk)
    (attend-feedback-NL isa chunk)
    (feedback-done isa chunk)
    (repetition-detection isa chunk)
    (attending-rd isa chunk)
    (start-add isa chunk)
    (one-ans isa chunk)
    (one-ans-borrow isa chunk)
    (one-ans-borrow2 isa chunk)
    (ten-ans-retrieve isa chunk)
    (ten-ans isa chunk)
    (read-base isa chunk)
    (dereferencing isa chunk)


    )

  ; for all non-math chunks
  (set-all-base-levels 1000 -1000)

  (add-weighted-addition-facts 1) ; wf
  

;;;; PERCEPTION ;;;;;


  (p look-for-base-jpb
     =goal>
      isa task
      state encode-base
      current-problem =prob
    ?imaginal>
      state free
   ==>
    +imaginal>
      isa pyramid
      current-problem =prob
    =goal>
        state findloc
        role base
    +visual-location>
        isa visual-location
        screen-x lowest
        screen-y lowest
  )

  (p look
    =goal>
      isa task
      state findloc
    =visual-location>
    ?manual>
       state free
    ?visual>
      state free
  ==>
    +visual>
      cmd move-attention
      screen-pos =visual-location
    +manual>
      isa move-cursor
      loc =visual-location
    =goal>
      state attend
   )
 
(p lookup-token
   =goal>
     isa task
     state attend
   =visual>
     isa text
     value =val
   ==>
  !eval! (string-to-chunk =val)
  +retrieval>
    isa equation-token
    string =val
   *goal>
     state dereferencing
  =visual>
 )


;; base

 (p attend-base
   =goal>
     isa task
     state dereferencing
   =visual>
     isa text
     value =val
     - value "x"
  =imaginal>
    isa pyramid
    base nil
  ?imaginal>
       state free
   =retrieval>
    ; isa equation-token
 ==>
   !safe-bind! =num (string-to-chunk =val)
   =goal>
     state findloc
   *imaginal>
     base =num
    +visual-location>
        isa visual-location
      > screen-x 125
      < screen-x 135
      < screen-y 80
 )
 
 (p attend-base-unknown
   =goal>
     isa task
     state dereferencing
   =visual>
     isa text
     value "x"
  =imaginal>
    isa pyramid
    base nil
  ?imaginal>
       state free
   =retrieval>
    ; isa equation-token
 ==>
   =goal>
     state findloc
  *imaginal>
     base X
     unknown-field base
    +visual-location>
        isa visual-location
      > screen-x 125
      < screen-x 135
      < screen-y 80
 )
 
;; height

 (p attend-height
   =goal>
     isa task
     state dereferencing
   =visual>
     isa text
     value =val
     - value "x"
  =imaginal>
    isa pyramid
    base =b
    height nil
  ?imaginal>
       state free
   =retrieval>
    ; isa equation-token
 ==>
   !safe-bind! =num (string-to-chunk =val)
   =goal>
     state findloc
   *imaginal>
     height =num
    +visual-location>
        isa visual-location
      > screen-x 175
      < screen-x 185
      < screen-y 80
 )
 
 (p attend-heigth-unknown
   =goal>
     isa task
     state dereferencing
   =visual>
     isa text
     value "x"
  =imaginal>
    isa pyramid
    base =b
    height nil
  ?imaginal>
       state free
   =retrieval>
    ; isa equation-token
 ==>
   =goal>
     state findloc
   *imaginal>
     height X
     unknown-field height
    +visual-location>
        isa visual-location
      > screen-x 175
      < screen-x 185
      < screen-y 80
 )
 
;; value

 (p attend-value
   =goal>
     isa task
     state dereferencing
   =visual>
     isa text
     value =val
     - value "x"
  =imaginal>
    isa pyramid
    base =b
    height =h
    value nil
  ?imaginal>
       state free
   =retrieval>
    ; isa equation-token
 ==>
   !safe-bind! =num (string-to-chunk =val)
    !safe-bind! =x (find-location-x "5")
    !safe-bind! =y (find-location-y "5")
   =goal>
     state begin-solving
     step ready
  *imaginal>
     value =num
  +visual-location> ;; look for the 5
    isa visual-location
    screen-x =x
    screen-y =y
 )
 
 (p attend-value-unknown
   =goal>
     isa task
     state dereferencing
   =visual>
     isa text
     value "x"
  =imaginal>
    isa pyramid
    base =b
    height =h
    value nil
  ?imaginal>
       state free
   =retrieval>
     ;isa equation-token
 ==>
    !safe-bind! =x (find-location-x "5")
    !safe-bind! =y (find-location-y "5")
   =goal>
     state begin-solving
     step ready
  *imaginal> ;;+
     value X
     unknown-field value
  +visual-location> ;; look for the 5
    isa visual-location
    screen-x =x
    screen-y =y
 )
 
;;;; PERCEPTION END ;;;;;


;;;; SOLVE-FOR-VALUE ;;;;;

(p begin-solving-uv-inplace
   =goal>
    isa task
    state begin-solving
  =imaginal>
    isa pyramid
    unknown-field value
    height =height
    base =base
  ?imaginal>
    state free
  ?manual>
    state free
  =visual-location>
   ; isa visual-location
  ==>
  !safe-bind! =baseOnes (ones =base)
  !safe-bind! =baseTens (tens =base)
  !safe-bind! =nextOnes (ones (decrement =base))
  !safe-bind! =nextTens (tens (decrement =base))
  !safe-bind! =rtd (decrement =height)
  +imaginal>
    isa add-pair
    one1 =baseOnes
    ten1 =baseTens
    one2 =nextOnes
    ten2 =nextTens
    scratch =rtd ; decrement each rep, stop when =0
  +manual> ; move cursor to number keys
    cmd move-cursor
    loc =visual-location
  *goal>
    state uv-in-place
    step start-add
    guess nil ; don't confuse with ub-as-uv
  )

 (p next-inplace-add-up
   =goal>
     isa task
     state uv-in-place
     step add-done
   =imaginal>
     isa add-pair
     combined-sum =sum
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch =rtd
  ?imaginal>
       state free
   !safe-eval! (not (eq =rtd 'n1))
   ==>
   !safe-bind! =last-term (combine =tens =ones)
   !safe-bind! =next-term (decrement =last-term)
   !safe-bind! =next-ones (ones =next-term)
   !safe-bind! =next-tens (tens =next-term)
   !safe-bind! =new-rtd (decrement =rtd)
   *imaginal>
     one-ans nil
     ten-ans nil
     combined-sum nil
     one1 =oa
     ten1 =ta
     one2 =next-ones
     ten2 =next-tens
     scratch =new-rtd
  +visual-location>
    isa visual-location
      > screen-x 125
      < screen-x 135
      < screen-y 80
  *goal>
    step start-add
 )
 
(p look-at-visloc
   =goal>
     isa task
     state uv-in-place
  =visual-location>
   ; isa visual-location
  ?visual>
    state free
==> 
  +visual>
    cmd move-attention
    screen-pos =visual-location
)

 (p uv-inplace-done
   =goal>
     isa task
     state uv-in-place
     step add-done
     guess nil ; not actually ub
   =imaginal>
     isa add-pair
     combined-sum =sum
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch n1 ; last rep done
   ?imaginal>
     state free
  ==>
  !safe-eval! (print (format nil "Answering ~A."  =sum))
  *goal>
    state answer
    step start
    value =sum
  +imaginal>
    isa respond
    value =sum
  )



 (p begin-solving-uh-simple
    =goal>
      isa task
      state begin-solving
    =imaginal>
      isa pyramid
      unknown-field height
      base =base
      value =value
   ?imaginal>
     state free
  ==>
  !safe-bind! =baseOnes (ones =base)
  !safe-bind! =baseTens (tens =base)
  !safe-bind! =nextOnes (ones (decrement =base))
  !safe-bind! =nextTens (tens (decrement =base))
  +imaginal>
    isa add-pair
    one1 =baseOnes
    ten1 =baseTens
    one2 =nextOnes
    ten2 =nextTens
    scratch =value ; scratch is the desired value
    scratch2 n2 ; scratch2 is the height for this answer
  *goal>
    state uh-simple
    step start-add
)

 (p next-uh-simple-add-up
   =goal>
     isa task
     state uh-simple
     step add-done
   =imaginal>
     isa add-pair
     combined-sum =sum
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch =val
     scratch2 =height
  ?imaginal>
       state free
   !safe-eval! (< (chunk-to-number =sum) (chunk-to-number =val))
   ==>
   !safe-bind! =last-term (combine =tens =ones)
   !safe-bind! =next-term (decrement =last-term)
   !safe-bind! =next-ones (ones =next-term)
   !safe-bind! =next-tens (tens =next-term)
   !safe-bind! =next-height (increment =height)
   *imaginal>
     one-ans nil
     ten-ans nil
     combined-sum nil
     one1 =oa
     ten1 =ta
     one2 =next-ones
     ten2 =next-tens
     scratch =val
     scratch2 =next-height
  +visual-location>
    isa visual-location
      > screen-x 175
      < screen-x 185
      < screen-y 80
  *goal>
    step start-add
 )
 
(p look-at-visloc2
   =goal>
     isa task
     state uh-simple
  =visual-location>
   ; isa visual-location
  ?visual>
   state free
==> 
  +visual>
    isa move-attention
    screen-pos =visual-location
)

 (p uh-simple-done
   =goal>
     isa task
     state uh-simple
     step add-done
   =imaginal>
     isa add-pair
     combined-sum =sum
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch =sum
     scratch2 =height
  ?imaginal>
    state free
  ==>
  !safe-eval! (print (format nil "Answering ~A."  =height))
  *goal>
    state answer
    step start
    value =height
  +imaginal>
    isa respond
    value =height
  )


 ; necessary to prevent infinite loops (don't go into negatives)
 (p uh-added-one-and-done
   =goal>
     isa task
     state uh-simple
     step add-done
   =imaginal>
     isa add-pair
     combined-sum =got
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch =desired-val
     scratch2 =height
     !safe-eval! (let ((val (chunk-to-number =got))
                     (desired-val (chunk-to-number =desired-val))
                     (current-term (chunk-to-number (combine =tens =ones))))
                 (and (not (= val desired-val))
                      (= current-term 1)))
  ?imaginal>
    state free
 ==>
  !safe-eval! (print (format nil "Answering ~A (added one and wrong)."  =height))
  +imaginal>
    isa respond
    value =height
  *goal>
    state answer
    step start
    value =height
  )
 
 ; ub is transformed to uv, with extra info on the goal
 (p begin-solving-ub
    =goal>
      isa task
      state begin-solving
    =imaginal>
      isa pyramid
      unknown-field base
      height =height
      value =value
   ?imaginal>
     state free
  ==>
  !safe-bind! =guess (guess  =height =value *guess-noise*)
  !safe-bind! =guessOnes (ones =guess)
  !safe-bind! =guessTens (tens =guess)
  !safe-bind! =nextOnes (ones (decrement =guess))
  !safe-bind! =nextTens (tens (decrement =guess))
  !safe-bind! =rtd (decrement =height)
  +imaginal>
    isa add-pair
    one1 =guessOnes
    ten1 =guessTens
    one2 =nextOnes
    ten2 =nextTens
    scratch =rtd
  *goal>
    state uv-in-place
    step start-add
    value =value
    guess =guess
  )
 
 (p ub-done-right 
   =goal>
     isa task
     state uv-in-place
     step add-done
     guess =guess
     value =sum
   =imaginal>
     isa add-pair
     combined-sum =sum
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch n1 ; last rep done
  ?imaginal>
   state free
  ==>
  !safe-eval! (print (format nil "Answering ~A."  =guess))
  +imaginal>
    isa respond
    value =guess
  *goal>
    state answer
    step start
    value =guess
  )


 (p unknown-base-too-large-assume-obo
    =goal>
     isa task
     state uv-in-place
     step add-done
     guess =base
     value =desired-val
   =imaginal>
     isa add-pair
     combined-sum =val-obtained
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch n1 ; last rep done
    !safe-eval! (> (chunk-to-number =val-obtained) (chunk-to-number =desired-val))
  ==>
    !safe-bind! =ans (decrement =base)    
    !safe-eval! (print (format nil "Answering ~A (obo too-large)."  =ans))
    *goal>
      state answer
      step start
      value =ans
 )
  
  (p unknown-base-too-small-assume-obo
    =goal>
     isa task
     state uv-in-place
     step add-done
     guess =base
     value =desired-val
   =imaginal>
     isa add-pair
     combined-sum =val-obtained
     one2 =ones
     ten2 =tens
     one-ans =oa
     ten-ans =ta
     scratch n1 ; last rep done
    !safe-eval! (< (chunk-to-number =val-obtained) (chunk-to-number =desired-val))
 
  ==>
    !safe-bind! =ans (increment =base)    
    !safe-eval! (print (format nil "Answering ~A (obo too-small)."  =ans))
    *goal>
      state answer
      step start
      value =ans
 )
 



;;; RESPOND ;;;;

 (p respond
     =goal>
       isa task
       state answer
       step start
       value =val
     
   ==>
   !safe-bind! =stringval (chunk-to-string =val)
     *goal>
       step respond ; press-keys
       value =stringval
)


 (p lookup-and-find-next-button-noret
    =goal>
      isa task
      step respond
      value =val
    ?visual>
      state free
    ?visual-location>
      state free
    !safe-eval! (and (not (equal =val "")) (not (equal (string (aref =val 0)) "/")))
  ==>
    !safe-bind! =rest (subseq =val 1 (length =val))
    !safe-bind! =curval (subseq =val 0 1)
    !safe-bind! =x (find-location-x =curval)
    !safe-bind! =y (find-location-y =curval)
    *goal>
      value =rest
      step prepare-click
    +visual-location>
      isa visual-location
      screen-x =x
      screen-y =y
  )

 (p attend-and-move-to-next-button
    =goal>
      isa task
      step prepare-click
    =visual-location>
     ; isa visual-location
    ?manual>
      state free
   ?visual>
     state free
  ==>
   !safe-eval! (setf *real-clicks* t)
    +visual>
      isa move-attention
      screen-pos =visual-location
    +manual>
      cmd move-cursor
      loc =visual-location
    *goal>
      step click
  )
  
(p click-next-button
    =goal>
      isa task
      step click
      value =val
    ?manual>
      state free
  ==>
    +manual>
      cmd click-mouse
    *goal>
      step attend-entered
 )
  
 (p attend-entered
   =goal>
    isa task
    step attend-entered
  =visual-location>
    isa visual-location
    color black
  ?visual>
    state free
 ==>
   +visual>
     isa move-attention
     screen-pos =visual-location
  *goal>
    step respond
 )

 (p lookup-and-find-submit-button-noret
    =goal>
      isa task
      step respond
      value ""
    ?visual>
      state free
    ?visual-location>
      state free
  ==>
    !bind! =x (find-location-x "submit")
    !bind! =y (find-location-y "submit")
    +visual-location>
      isa visual-location
      screen-x =x
      screen-y =y
    *goal>
      step prepare-click-s
  )
 
(p attend-and-move-to-submit
   =goal>
     isa task
     step prepare-click-s
   =visual-location>
    ; isa visual-location
   ?manual>
     state free
  ?visual>
    state free
 ==>
   +visual>
     isa move-attention
     screen-pos =visual-location
   +manual>
     isa move-cursor
     loc =visual-location
   *goal>
     step click-s
 )

 (p click-submit
    =goal>
      isa task
      step click-s
      value =val
    ?manual>
      state free
  ==>
    +manual>
      cmd click-mouse
    *goal>
      step ready
      state done
 )
   
   (p done
     =goal>
       isa task
       state done
    ?visual>
      state free
   ==>
     *goal>
       step find-feedback ;attend-feedback
       state read-base
     +visual>
      isa clear
   )
 
;;;FEEDBACK 

 (p find-feedback
  =goal>
    isa task
    step find-feedback
  =visual> ; anything
    isa text
    color red ; only read red things
  ?visual-location>
    state free
  ?visual>
    state free
  ?imaginal>
    state free
  ==>
  +imaginal>
    isa feedback
    +visual-location>
      isa visual-location
      screen-y current
      screen-x lowest
      > screen-x current
  )
 
(p dont-attend-feedback
 =goal>
   isa task
   step find-feedback
 =visual> ; anything
   isa text
   - color red ; only read red things
 ==>
  *goal>
    step feedback-done 
 )
 
 (p find-feedback-next-line
  =goal>
    isa task
    step find-feedback
  ?visual-location>
   state error
  ==>
    +visual-location>
      isa visual-location
      screen-y lowest
      screen-x lowest
      > screen-y current
   *goal>
     step attend-feedback-nl ; can't stay attend-feedback or it will loop on error
  )


 (p attend-feedback
    =goal>
     isa task
     step find-feedback
   =visual-location>
  ;   isa visual-location
   ?visual>
     state free
 ==>
   +visual>
     isa move-attention
     screen-pos =visual-location
 )
 

 (p attend-feedback-nl
    =goal>
     isa task
     step attend-feedback-nl
   =visual-location>
   ;  isa visual-location
   ?visual>
     state free
 ==>
   +visual>
     isa move-attention
     screen-pos =visual-location
  *goal>
    step find-feedback
 )

 (p feedback-done
    =goal>
     isa task
     step attend-feedback-nl
   ?visual-location>
     state error
   ?visual>
     state free
 ==>
  *goal>
    step feedback-done
)



 ; ; addition productions adapted from tutorial model
   ; subtraction capability included, in that case the second number should be
   ; negated (both digits)
   ;
   ; assuming the following are done in rule conditions, rather than through
   ; fact retrieval:
   ;
   ; dividing out and recombining the ones and tens places
   ; checking if a number is > 10
   ; checking if a number is < 0
   ; adding 10 to a number 0-9 (unless that is the overall problem)
   ; subtracting 10 from 10-19 (similarly)
   ;
   ; interface: build an imaginal chunk for the addition problem
   ; set iteration to be the current iteration chunk (which will be restored to
   ; the buffer when done), set role to be the slot name in the iteration chunk
   ; where the result should be placed, and next-step to be the goal step to set
   ; when its all done (at which point the imaginal chunk for the iteration will
   ; be back and have its value)

   (p addition-start-pair
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 =ten1
       ten2 =ten2
       one-ans nil
     ?retrieval>
       state free
    =goal>
      isa task
      step start-add
   ==>
      =imaginal> =imaginal
     *goal>
       step one-ans
     +retrieval>
       isa addition-fact
       addend1 =one1
       addend2 =one2
   )

   ;;2+3
   (p add-ones-no-carry-no-tens-done
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 n0
       ten2 n0
    =goal>
      isa task
      step one-ans
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
     !safe-eval! (and (< (chunk-to-number =sum) 10) (>= (chunk-to-number =sum) 0))
   ==>
     *imaginal>
       one-ans =sum
       ten-ans n0
       combined-sum =sum
       carry nil 
    *goal>
      step add-done
   )

  ;; 5+12
   (p add-ones-no-carry-one-ten2
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 n0
       ten2 =ten2
     - ten2 n0
    =goal>
      isa task
      step one-ans
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
     !safe-eval! (and (< (chunk-to-number =sum) 10) (>= (chunk-to-number =sum) 0))
   ==>
     *imaginal>
       one-ans =sum
       ten1 =ten2
       ten2 n0
       carry nil 
    *goal>
      step ten-add
   )
  ;; 15+2
   (p add-ones-no-carry-one-ten1
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 =ten1
     - ten1 n0
       ten2 n0
    =goal>
      isa task
      step one-ans
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
     !safe-eval! (and (< (chunk-to-number =sum) 10) (>= (chunk-to-number =sum) 0))
   ==>
     *imaginal>
       one-ans =sum
       carry nil 
    *goal>
      step ten-add
   )

   ;; 15+12
   (p add-ones-no-carry-tens
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 =ten1
     - ten1 n0 
       ten2 =ten2
     - ten2 n0
    =goal>
      isa task
      step one-ans
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
     !safe-eval! (and (< (chunk-to-number =sum) 10) (>= (chunk-to-number =sum) 0))
   ==>
     *imaginal>
       one-ans =sum
       carry nil 
     +retrieval>
       isa addition-fact
       addend1 =ten1 
       addend2 =ten2 
    *goal>
      step ten-ans
   )


   ;; 5+7
   (p add-ones-and-carry-no-tens-done
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 n0
       ten2 n0
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
    =goal>
      isa task
      step one-ans
     !safe-eval! (>= (chunk-to-number =sum) 10)
   ==>
     !safe-bind! =remainder (tostring (- (chunk-to-number =sum) 10))
     *imaginal>
       one-ans =remainder
       carry n1
    *goal>
      step add-carry
   )

 ;;5+17
   (p add-ones-and-carry-one-ten2
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 n0
       ten2 =ten2
     - ten2 n0
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
    =goal>
      isa task
      step one-ans
      !safe-eval! (>= (chunk-to-number =sum) 10)
   ==>
     !safe-bind! =remainder (tostring (- (chunk-to-number =sum) 10))
     *imaginal>
       one-ans =remainder
       ten1 =ten2
       ten2 n0
       carry n1
    *goal>
      step ten-add
   )
 
   ;;15+7
   (p add-ones-and-carry-one-ten1
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 =ten1
     - ten1 n0 
       ten2 n0
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
    =goal>
      isa task
      step one-ans
      !safe-eval! (>= (chunk-to-number =sum) 10)
   ==>
     !safe-bind! =remainder (tostring (- (chunk-to-number =sum) 10))
     *imaginal>
       one-ans =remainder
       carry n1
    *goal>
      step ten-add
   )

   ;;15+17
   (p add-ones-and-carry-tens
     =imaginal>
       isa add-pair
       one1 =one1
       one2 =one2
       ten1 =ten1
     - ten1 n0 
       ten2 =ten2
     - ten2 n0
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
    =goal>
      isa task
      step one-ans
      !safe-eval! (>= (chunk-to-number =sum) 10)
   ==>
     !safe-bind! =remainder (tostring (- (chunk-to-number =sum) 10))
     +retrieval>
       isa addition-fact
       addend1 =ten1 
       addend2 =ten2 
     *imaginal>
       one-ans =remainder
       carry n1
    *goal>
      step ten-ans
   )




(p no-tens-carry-done
     =goal>
       isa task
       step add-carry
     =imaginal>
       isa add-pair
       one-ans =ones
       carry n1
     ?imaginal>
       state free
   ==>
     !safe-bind! =comb (combine 'n1 =ones)
     *imaginal>
       ten-ans n1
       combined-sum =comb
    *goal>
      step add-done
   )


 (p add-tens-carry
   =imaginal>
     isa add-pair
     ten1 =ten1
     ten2 =ten2
    carry =carry
   =goal>
      isa task
      step ten-ans
    =retrieval>
     isa addition-fact
     sum =sum
  ?imaginal>
       state free
 ==>
   *imaginal>
     carry nil
   +retrieval>
     isa addition-fact
     addend1 =carry
     addend2 =sum
 )

 (p add-one-ten-carry
   =imaginal>
     isa add-pair
     ten1 =ten1
     ten2 n0
    carry =carry
   =goal>
      isa task
      step ten-add
  ?imaginal>
       state free
 ==>
   *imaginal>
     carry nil
   +retrieval>
     isa addition-fact
     addend1 =carry
     addend2 =ten1
   *goal>
     step ten-ans
 )



   (p add-tens-done
     =goal>
       isa task
       current-problem =prob
      step ten-ans
     =imaginal>
       isa add-pair
       one-ans =ones
       carry nil
     =retrieval>
       isa addition-fact
       sum =sum
     ?imaginal>
       state free
   ==>
     !safe-bind! =comb (combine =sum =ones)
     *imaginal>
       ten-ans =sum
       combined-sum =comb
    *goal>
      step add-done
   )

 (p add-one-ten-done
     =goal>
       isa task
       current-problem =prob
      step ten-add
     =imaginal>
       isa add-pair
       one-ans =ones
       ten1 =ten1
       ten2 n0
       carry nil
    ?imaginal>
       state free
   ==>
     !safe-bind! =comb (combine =ten1 =ones)
     *imaginal>
       ten-ans =ten1
       combined-sum =comb
    *goal>
      step add-done
   )

    (goal-focus goal)
  )



;; Definition of actual problems seen by the subjects
(setf *real-problems* '(
( 1 (
(
( 6 $ 2 = 11 Small Value )
( 5 $ 2 = 9 Small Base )
( 8 $ 3 = 21 Small Value )
( 8 $ 2 = 15 Small Height )
( 4 $ 3 = 9 Small Height )
( 5 $ 4 = 14 Large Value )
( 6 $ 4 = 18 Large Height )
( 7 $ 5 = 25 Large Value )
( 6 $ 2 = 11 Small Base )
( 8 $ 5 = 30 Large Height )
( 4 $ 5 = 10 Large Base )
)
(
( 7 $ 3 = 18 Small Height )
( 8 $ 4 = 26 Large Base )
( 8 $ 5 = 30 Large Base )
( 4 $ 2 = 7 Small Base )
( 9 $ 2 = 17 Small Base )
( 5 $ 4 = 14 Large Height )
( 4 $ 5 = 10 Large Value )
( 4 $ 3 = 9 Small Value )
( 5 $ 5 = 15 Large Height )
( 6 $ 4 = 18 Large Value )
)
(
( 7 $ 4 = 22 Large Height )
( 8 $ 3 = 21 Small Height )
( 4 $ 2 = 7 Small Value )
( 9 $ 5 = 35 Large Height )
( 6 $ 4 = 18 Large Base )
( 9 $ 2 = 17 Small Height )
( 9 $ 4 = 30 Large Base )
( 6 $ 5 = 20 Large Value )
( 4 $ 3 = 9 Small Base )
( 7 $ 3 = 18 Small Base )
( 8 $ 4 = 26 Large Value )
)
(
( 9 $ 5 = 35 Large Value )
( 6 $ 5 = 20 Large Height )
( 9 $ 4 = 30 Large Height )
( 9 $ 3 = 24 Small Value )
( 7 $ 2 = 13 Small Base )
( 5 $ 2 = 9 Small Height )
( 8 $ 2 = 15 Small Value )
( 8 $ 2 = 15 Small Base )
( 7 $ 5 = 25 Large Base )
( 8 $ 5 = 30 Large Value )
)
(
( 4 $ 4 = 10 Large Base )
( 4 $ 5 = 10 Large Height )
( 6 $ 2 = 11 Small Height )
( 9 $ 4 = 30 Large Value )
( 5 $ 3 = 12 Small Base )
( 6 $ 3 = 15 Small Value )
( 4 $ 4 = 10 Large Height )
( 9 $ 3 = 24 Small Height )
( 4 $ 4 = 10 Large Value )
( 5 $ 3 = 12 Small Value )
( 8 $ 3 = 21 Small Base )
)
(
( 7 $ 3 = 18 Small Value )
( 6 $ 3 = 15 Small Base )
( 7 $ 2 = 13 Small Height )
( 6 $ 5 = 20 Large Base )
( 4 $ 2 = 7 Small Height )
( 7 $ 5 = 25 Large Height )
( 7 $ 4 = 22 Large Value )
( 5 $ 2 = 9 Small Value )
( 5 $ 5 = 15 Large Value )
( 9 $ 3 = 24 Small Base )
)
))
( 2 (
(
( 4 $ 3 = 9 Small Height )
( 4 $ 2 = 7 Small Value )
( 5 $ 3 = 12 Small Height )
( 6 $ 2 = 11 Small Base )
( 9 $ 5 = 35 Large Base )
( 9 $ 4 = 30 Large Value )
( 5 $ 5 = 15 Large Height )
( 4 $ 3 = 9 Small Value )
)
(
( 6 $ 5 = 20 Large Height )
( 5 $ 5 = 15 Large Value )
( 6 $ 2 = 11 Small Value )
( 7 $ 2 = 13 Small Base )
( 5 $ 4 = 14 Large Base )
( 4 $ 4 = 10 Large Value )
( 8 $ 2 = 15 Small Base )
( 5 $ 2 = 9 Small Height )
( 9 $ 5 = 35 Large Height )
( 4 $ 5 = 10 Large Base )
( 7 $ 3 = 18 Small Value )
)
(
( 8 $ 3 = 21 Small Height )
( 4 $ 3 = 9 Small Base )
( 5 $ 5 = 15 Large Base )
( 9 $ 2 = 17 Small Base )
( 8 $ 4 = 26 Large Base )
( 7 $ 4 = 22 Large Height )
( 9 $ 4 = 30 Large Height )
( 6 $ 3 = 15 Small Height )
( 5 $ 2 = 9 Small Value )
( 8 $ 4 = 26 Large Value )
( 6 $ 3 = 15 Small Value )
( 9 $ 5 = 35 Large Value )
)
(
( 8 $ 2 = 15 Small Height )
( 4 $ 4 = 10 Large Height )
( 9 $ 4 = 30 Large Base )
( 7 $ 4 = 22 Large Value )
( 8 $ 4 = 26 Large Height )
( 7 $ 3 = 18 Small Base )
( 9 $ 3 = 24 Small Height )
( 4 $ 4 = 10 Large Base )
( 8 $ 2 = 15 Small Value )
( 9 $ 3 = 24 Small Base )
)
(
( 6 $ 4 = 18 Large Base )
( 9 $ 2 = 17 Small Value )
( 9 $ 2 = 17 Small Height )
( 8 $ 3 = 21 Small Value )
( 5 $ 2 = 9 Small Base )
( 6 $ 2 = 11 Small Height )
( 4 $ 2 = 7 Small Base )
( 4 $ 5 = 10 Large Value )
( 4 $ 5 = 10 Large Height )
( 7 $ 4 = 22 Large Base )
)
(
( 6 $ 4 = 18 Large Height )
( 6 $ 3 = 15 Small Base )
( 6 $ 5 = 20 Large Base )
( 4 $ 2 = 7 Small Height )
( 7 $ 5 = 25 Large Value )
( 8 $ 5 = 30 Large Base )
( 9 $ 3 = 24 Small Value )
( 8 $ 5 = 30 Large Value )
( 7 $ 2 = 13 Small Value )
( 5 $ 3 = 12 Small Base )
( 7 $ 5 = 25 Large Height )
( 7 $ 2 = 13 Small Height )
)
))
( 3 (
(
( 5 $ 4 = 14 Large Base )
( 5 $ 5 = 15 Large Value )
( 6 $ 2 = 11 Small Height )
( 9 $ 3 = 24 Small Value )
( 9 $ 2 = 17 Small Value )
( 8 $ 4 = 26 Large Height )
( 7 $ 3 = 18 Small Base )
( 5 $ 2 = 9 Small Base )
( 7 $ 4 = 22 Large Height )
( 6 $ 4 = 18 Large Base )
)
(
( 4 $ 2 = 7 Small Height )
( 6 $ 3 = 15 Small Value )
( 4 $ 5 = 10 Large Base )
( 5 $ 5 = 15 Large Height )
( 9 $ 2 = 17 Small Base )
( 8 $ 2 = 15 Small Height )
( 8 $ 3 = 21 Small Value )
( 9 $ 5 = 35 Large Value )
( 6 $ 3 = 15 Small Base )
( 6 $ 5 = 20 Large Height )
)
(
( 4 $ 3 = 9 Small Height )
( 7 $ 3 = 18 Small Value )
( 8 $ 5 = 30 Large Base )
( 9 $ 5 = 35 Large Height )
( 4 $ 2 = 7 Small Base )
( 4 $ 3 = 9 Small Base )
( 6 $ 5 = 20 Large Base )
( 5 $ 2 = 9 Small Height )
( 6 $ 2 = 11 Small Value )
( 8 $ 5 = 30 Large Value )
)
(
( 6 $ 2 = 11 Small Base )
( 4 $ 4 = 10 Large Base )
( 5 $ 2 = 9 Small Value )
( 9 $ 5 = 35 Large Base )
( 7 $ 3 = 18 Small Height )
( 4 $ 5 = 10 Large Height )
( 6 $ 4 = 18 Large Height )
( 7 $ 5 = 25 Large Value )
( 8 $ 3 = 21 Small Height )
)
(
( 4 $ 4 = 10 Large Value )
( 8 $ 2 = 15 Small Base )
( 5 $ 4 = 14 Large Height )
( 7 $ 5 = 25 Large Height )
( 5 $ 3 = 12 Small Height )
( 4 $ 2 = 7 Small Value )
( 5 $ 3 = 12 Small Base )
( 5 $ 5 = 15 Large Base )
( 4 $ 5 = 10 Large Value )
( 7 $ 2 = 13 Small Value )
)
(
( 8 $ 5 = 30 Large Height )
( 8 $ 3 = 21 Small Base )
( 7 $ 5 = 25 Large Base )
( 9 $ 2 = 17 Small Height )
( 4 $ 3 = 9 Small Value )
( 6 $ 4 = 18 Large Value )
( 5 $ 3 = 12 Small Value )
( 9 $ 4 = 30 Large Value )
( 7 $ 2 = 13 Small Base )
( 7 $ 2 = 13 Small Height )
)
))
( 4 (
(
( 4 $ 3 = 9 Small Height )
( 9 $ 3 = 24 Small Value )
( 6 $ 4 = 18 Large Value )
( 8 $ 4 = 26 Large Base )
( 6 $ 2 = 11 Small Value )
( 6 $ 4 = 18 Large Base )
( 8 $ 5 = 30 Large Height )
( 9 $ 2 = 17 Small Height )
( 9 $ 4 = 30 Large Value )
)
(
( 4 $ 5 = 10 Large Base )
( 8 $ 2 = 15 Small Height )
( 9 $ 3 = 24 Small Base )
( 4 $ 5 = 10 Large Height )
( 5 $ 3 = 12 Small Value )
( 8 $ 4 = 26 Large Value )
( 7 $ 2 = 13 Small Value )
( 7 $ 5 = 25 Large Height )
( 8 $ 3 = 21 Small Base )
)
(
( 7 $ 3 = 18 Small Value )
( 5 $ 4 = 14 Large Base )
( 5 $ 2 = 9 Small Height )
( 5 $ 4 = 14 Large Height )
( 7 $ 5 = 25 Large Base )
( 9 $ 2 = 17 Small Base )
( 9 $ 2 = 17 Small Value )
( 4 $ 4 = 10 Large Value )
( 8 $ 5 = 30 Large Value )
( 9 $ 3 = 24 Small Height )
( 8 $ 2 = 15 Small Base )
( 6 $ 4 = 18 Large Height )
)
(
( 5 $ 5 = 15 Large Base )
( 7 $ 5 = 25 Large Value )
( 8 $ 2 = 15 Small Value )
( 4 $ 4 = 10 Large Height )
( 4 $ 2 = 7 Small Base )
( 6 $ 3 = 15 Small Height )
( 9 $ 5 = 35 Large Base )
( 8 $ 3 = 21 Small Value )
( 5 $ 5 = 15 Large Value )
( 5 $ 2 = 9 Small Base )
( 7 $ 4 = 22 Large Height )
( 7 $ 2 = 13 Small Height )
)
(
( 7 $ 4 = 22 Large Value )
( 6 $ 2 = 11 Small Base )
( 4 $ 2 = 7 Small Value )
( 6 $ 3 = 15 Small Value )
( 7 $ 4 = 22 Large Base )
( 6 $ 2 = 11 Small Height )
( 5 $ 3 = 12 Small Base )
( 5 $ 4 = 14 Large Value )
( 6 $ 5 = 20 Large Height )
( 9 $ 4 = 30 Large Height )
( 8 $ 3 = 21 Small Height )
)
(
( 5 $ 2 = 9 Small Value )
( 5 $ 3 = 12 Small Height )
( 5 $ 5 = 15 Large Height )
( 8 $ 4 = 26 Large Height )
( 7 $ 2 = 13 Small Base )
( 4 $ 5 = 10 Large Value )
( 4 $ 3 = 9 Small Value )
( 4 $ 4 = 10 Large Base )
( 9 $ 5 = 35 Large Value )
( 7 $ 3 = 18 Small Height )
)
))
( 5 (
(
( 8 $ 5 = 30 Large Value )
( 9 $ 2 = 17 Small Value )
( 4 $ 4 = 10 Large Value )
( 5 $ 3 = 12 Small Base )
( 7 $ 2 = 13 Small Height )
( 9 $ 4 = 30 Large Height )
( 5 $ 2 = 9 Small Height )
( 5 $ 4 = 14 Large Height )
( 9 $ 3 = 24 Small Value )
( 6 $ 4 = 18 Large Base )
( 9 $ 5 = 35 Large Base )
)
(
( 9 $ 3 = 24 Small Base )
( 6 $ 2 = 11 Small Height )
( 8 $ 4 = 26 Large Value )
( 8 $ 2 = 15 Small Base )
( 4 $ 4 = 10 Large Height )
( 9 $ 4 = 30 Large Value )
( 8 $ 3 = 21 Small Value )
( 7 $ 5 = 25 Large Base )
( 9 $ 4 = 30 Large Base )
( 5 $ 3 = 12 Small Value )
( 7 $ 3 = 18 Small Height )
)
(
( 4 $ 4 = 10 Large Base )
( 4 $ 5 = 10 Large Height )
( 8 $ 5 = 30 Large Base )
( 7 $ 2 = 13 Small Value )
( 6 $ 4 = 18 Large Value )
( 5 $ 3 = 12 Small Height )
( 6 $ 5 = 20 Large Height )
( 8 $ 3 = 21 Small Height )
( 7 $ 2 = 13 Small Base )
( 6 $ 3 = 15 Small Value )
( 4 $ 5 = 10 Large Value )
)
(
( 6 $ 3 = 15 Small Height )
( 4 $ 2 = 7 Small Base )
( 5 $ 5 = 15 Large Value )
( 5 $ 2 = 9 Small Base )
( 9 $ 2 = 17 Small Height )
( 8 $ 4 = 26 Large Base )
( 9 $ 5 = 35 Large Value )
( 8 $ 2 = 15 Small Value )
( 5 $ 5 = 15 Large Height )
( 9 $ 5 = 35 Large Height )
( 4 $ 3 = 9 Small Value )
( 7 $ 4 = 22 Large Base )
)
(
( 4 $ 3 = 9 Small Base )
( 7 $ 5 = 25 Large Value )
( 8 $ 4 = 26 Large Height )
( 4 $ 2 = 7 Small Height )
( 4 $ 5 = 10 Large Base )
( 7 $ 5 = 25 Large Height )
( 9 $ 3 = 24 Small Height )
( 5 $ 4 = 14 Large Value )
( 9 $ 2 = 17 Small Base )
( 5 $ 2 = 9 Small Value )
( 6 $ 2 = 11 Small Value )
)
(
( 6 $ 4 = 18 Large Height )
( 5 $ 5 = 15 Large Base )
( 6 $ 5 = 20 Large Value )
( 4 $ 3 = 9 Small Height )
( 7 $ 3 = 18 Small Value )
( 7 $ 4 = 22 Large Value )
( 6 $ 2 = 11 Small Base )
( 4 $ 2 = 7 Small Value )
( 8 $ 2 = 15 Small Height )
( 7 $ 4 = 22 Large Height )
( 6 $ 5 = 20 Large Base )
( 7 $ 3 = 18 Small Base )
)
))
( 6 (
(
( 9 $ 3 = 24 Small Height )
( 8 $ 5 = 30 Large Value )
( 5 $ 5 = 15 Large Base )
( 9 $ 2 = 17 Small Base )
( 9 $ 4 = 30 Large Value )
( 7 $ 2 = 13 Small Height )
( 7 $ 4 = 22 Large Base )
( 6 $ 4 = 18 Large Height )
( 7 $ 2 = 13 Small Value )
)
(
( 8 $ 3 = 21 Small Height )
( 4 $ 4 = 10 Large Base )
( 4 $ 5 = 10 Large Height )
( 7 $ 4 = 22 Large Value )
( 4 $ 5 = 10 Large Base )
( 7 $ 3 = 18 Small Base )
( 9 $ 5 = 35 Large Height )
( 6 $ 3 = 15 Small Value )
( 8 $ 4 = 26 Large Value )
( 9 $ 2 = 17 Small Height )
( 4 $ 2 = 7 Small Base )
( 7 $ 3 = 18 Small Value )
)
(
( 8 $ 2 = 15 Small Height )
( 4 $ 4 = 10 Large Value )
( 9 $ 5 = 35 Large Base )
( 8 $ 4 = 26 Large Height )
( 7 $ 2 = 13 Small Base )
( 4 $ 2 = 7 Small Value )
( 7 $ 4 = 22 Large Height )
( 9 $ 4 = 30 Large Base )
( 4 $ 2 = 7 Small Height )
( 9 $ 2 = 17 Small Value )
( 9 $ 3 = 24 Small Base )
)
(
( 5 $ 4 = 14 Large Base )
( 5 $ 5 = 15 Large Value )
( 6 $ 5 = 20 Large Value )
( 8 $ 5 = 30 Large Base )
( 8 $ 5 = 30 Large Height )
( 5 $ 2 = 9 Small Value )
( 5 $ 2 = 9 Small Base )
( 6 $ 5 = 20 Large Height )
( 7 $ 3 = 18 Small Height )
( 9 $ 3 = 24 Small Value )
)
(
( 5 $ 2 = 9 Small Height )
( 6 $ 5 = 20 Large Base )
( 6 $ 4 = 18 Large Value )
( 5 $ 3 = 12 Small Height )
( 8 $ 2 = 15 Small Value )
( 5 $ 4 = 14 Large Value )
( 6 $ 2 = 11 Small Base )
( 5 $ 4 = 14 Large Height )
( 6 $ 2 = 11 Small Value )
( 7 $ 5 = 25 Large Height )
( 8 $ 2 = 15 Small Base )
)
(
( 8 $ 3 = 21 Small Value )
( 6 $ 2 = 11 Small Height )
( 4 $ 3 = 9 Small Base )
( 5 $ 3 = 12 Small Base )
( 4 $ 5 = 10 Large Value )
( 6 $ 4 = 18 Large Base )
( 7 $ 5 = 25 Large Value )
( 4 $ 4 = 10 Large Height )
( 6 $ 3 = 15 Small Height )
( 4 $ 3 = 9 Small Value )
( 7 $ 5 = 25 Large Base )
( 5 $ 5 = 15 Large Height )
)
))
( 7 (
(
( 6 $ 3 = 15 Small Height )
( 4 $ 4 = 10 Large Value )
( 5 $ 2 = 9 Small Base )
( 7 $ 5 = 25 Large Base )
( 8 $ 3 = 21 Small Value )
( 9 $ 3 = 24 Small Height )
( 8 $ 4 = 26 Large Height )
( 9 $ 4 = 30 Large Height )
( 5 $ 5 = 15 Large Base )
( 6 $ 2 = 11 Small Base )
( 9 $ 2 = 17 Small Value )
( 7 $ 5 = 25 Large Value )
)
(
( 6 $ 5 = 20 Large Base )
( 4 $ 4 = 10 Large Base )
( 6 $ 2 = 11 Small Value )
( 8 $ 5 = 30 Large Value )
( 6 $ 3 = 15 Small Base )
( 9 $ 5 = 35 Large Value )
( 4 $ 3 = 9 Small Height )
( 9 $ 3 = 24 Small Base )
( 6 $ 4 = 18 Large Height )
( 4 $ 3 = 9 Small Value )
)
(
( 7 $ 4 = 22 Large Height )
( 6 $ 4 = 18 Large Value )
( 7 $ 3 = 18 Small Base )
( 7 $ 2 = 13 Small Height )
( 8 $ 2 = 15 Small Base )
( 7 $ 4 = 22 Large Base )
( 5 $ 2 = 9 Small Value )
( 8 $ 4 = 26 Large Value )
( 9 $ 5 = 35 Large Base )
( 6 $ 3 = 15 Small Value )
( 8 $ 2 = 15 Small Height )
)
(
( 5 $ 5 = 15 Large Value )
( 6 $ 2 = 11 Small Height )
( 6 $ 5 = 20 Large Height )
( 7 $ 2 = 13 Small Value )
( 4 $ 2 = 7 Small Base )
( 7 $ 2 = 13 Small Base )
( 9 $ 4 = 30 Large Base )
( 4 $ 5 = 10 Large Base )
( 4 $ 5 = 10 Large Height )
( 5 $ 3 = 12 Small Value )
( 9 $ 2 = 17 Small Height )
( 9 $ 4 = 30 Large Value )
)
(
( 9 $ 3 = 24 Small Value )
( 8 $ 3 = 21 Small Base )
( 4 $ 2 = 7 Small Value )
( 8 $ 5 = 30 Large Base )
( 4 $ 2 = 7 Small Height )
( 6 $ 5 = 20 Large Value )
( 8 $ 3 = 21 Small Height )
( 5 $ 4 = 14 Large Value )
( 4 $ 4 = 10 Large Height )
( 6 $ 4 = 18 Large Base )
)
(
( 9 $ 2 = 17 Small Base )
( 5 $ 4 = 14 Large Base )
( 7 $ 5 = 25 Large Height )
( 8 $ 4 = 26 Large Base )
( 4 $ 3 = 9 Small Base )
( 4 $ 5 = 10 Large Value )
( 7 $ 4 = 22 Large Value )
( 8 $ 2 = 15 Small Value )
( 8 $ 5 = 30 Large Height )
( 5 $ 2 = 9 Small Height )
( 7 $ 3 = 18 Small Value )
( 5 $ 3 = 12 Small Height )
)
))
( 8 (
(
( 5 $ 3 = 12 Small Base )
( 5 $ 2 = 9 Small Height )
( 8 $ 4 = 26 Large Height )
( 9 $ 4 = 30 Large Base )
( 4 $ 2 = 7 Small Value )
( 4 $ 4 = 10 Large Value )
( 4 $ 2 = 7 Small Base )
( 8 $ 3 = 21 Small Height )
( 4 $ 5 = 10 Large Value )
( 8 $ 3 = 21 Small Value )
( 6 $ 4 = 18 Large Base )
( 5 $ 5 = 15 Large Height )
)
(
( 7 $ 2 = 13 Small Base )
( 5 $ 2 = 9 Small Value )
( 7 $ 5 = 25 Large Base )
( 5 $ 5 = 15 Large Value )
( 5 $ 4 = 14 Large Height )
( 9 $ 2 = 17 Small Value )
( 8 $ 5 = 30 Large Base )
( 7 $ 3 = 18 Small Base )
)
(
( 5 $ 2 = 9 Small Base )
( 8 $ 2 = 15 Small Height )
( 7 $ 3 = 18 Small Height )
( 8 $ 2 = 15 Small Base )
( 7 $ 2 = 13 Small Value )
( 9 $ 5 = 35 Large Height )
( 9 $ 5 = 35 Large Value )
( 9 $ 4 = 30 Large Height )
( 6 $ 4 = 18 Large Value )
)
(
( 9 $ 2 = 17 Small Height )
( 5 $ 4 = 14 Large Value )
( 8 $ 2 = 15 Small Value )
( 8 $ 3 = 21 Small Base )
( 9 $ 5 = 35 Large Base )
( 4 $ 5 = 10 Large Height )
( 4 $ 5 = 10 Large Base )
( 4 $ 4 = 10 Large Height )
( 9 $ 3 = 24 Small Base )
( 6 $ 3 = 15 Small Height )
)
(
( 8 $ 4 = 26 Large Base )
( 8 $ 4 = 26 Large Value )
( 4 $ 2 = 7 Small Height )
( 7 $ 5 = 25 Large Height )
( 6 $ 3 = 15 Small Value )
( 9 $ 3 = 24 Small Value )
( 8 $ 5 = 30 Large Height )
( 4 $ 3 = 9 Small Base )
( 7 $ 2 = 13 Small Height )
( 6 $ 3 = 15 Small Base )
( 5 $ 4 = 14 Large Base )
)
(
( 5 $ 5 = 15 Large Base )
( 6 $ 2 = 11 Small Height )
( 4 $ 4 = 10 Large Base )
( 4 $ 3 = 9 Small Value )
( 7 $ 4 = 22 Large Height )
( 6 $ 4 = 18 Large Height )
( 6 $ 2 = 11 Small Base )
( 8 $ 5 = 30 Large Value )
( 6 $ 5 = 20 Large Value )
( 4 $ 3 = 9 Small Height )
( 9 $ 2 = 17 Small Base )
( 6 $ 2 = 11 Small Value )
)
))
( 9 (
(
( 9 $ 4 = 30 Large Height )
( 6 $ 4 = 18 Large Value )
( 4 $ 3 = 9 Small Base )
( 5 $ 4 = 14 Large Base )
( 9 $ 2 = 17 Small Height )
( 6 $ 4 = 18 Large Base )
( 5 $ 5 = 15 Large Value )
( 4 $ 3 = 9 Small Height )
( 8 $ 2 = 15 Small Value )
( 6 $ 3 = 15 Small Value )
( 8 $ 4 = 26 Large Height )
( 5 $ 3 = 12 Small Base )
)
(
( 9 $ 5 = 35 Large Height )
( 4 $ 4 = 10 Large Value )
( 9 $ 5 = 35 Large Base )
( 9 $ 4 = 30 Large Value )
( 5 $ 2 = 9 Small Value )
( 8 $ 3 = 21 Small Value )
( 9 $ 3 = 24 Small Height )
( 6 $ 2 = 11 Small Base )
( 8 $ 5 = 30 Large Base )
( 4 $ 2 = 7 Small Height )
)
(
( 9 $ 2 = 17 Small Value )
( 7 $ 5 = 25 Large Base )
( 8 $ 3 = 21 Small Height )
( 8 $ 3 = 21 Small Base )
( 7 $ 4 = 22 Large Value )
( 9 $ 2 = 17 Small Base )
( 5 $ 4 = 14 Large Height )
( 5 $ 5 = 15 Large Base )
( 8 $ 2 = 15 Small Height )
( 7 $ 5 = 25 Large Value )
( 4 $ 4 = 10 Large Height )
( 6 $ 2 = 11 Small Value )
)
(
( 7 $ 2 = 13 Small Height )
( 7 $ 5 = 25 Large Height )
( 4 $ 4 = 10 Large Base )
( 7 $ 4 = 22 Large Height )
( 6 $ 5 = 20 Large Base )
( 4 $ 3 = 9 Small Value )
( 5 $ 2 = 9 Small Base )
( 6 $ 2 = 11 Small Height )
( 5 $ 4 = 14 Large Value )
( 6 $ 5 = 20 Large Value )
( 7 $ 2 = 13 Small Value )
( 8 $ 2 = 15 Small Base )
)
(
( 5 $ 3 = 12 Small Height )
( 6 $ 5 = 20 Large Height )
( 5 $ 2 = 9 Small Height )
( 7 $ 3 = 18 Small Value )
( 7 $ 4 = 22 Large Base )
( 4 $ 2 = 7 Small Base )
( 7 $ 2 = 13 Small Base )
( 9 $ 3 = 24 Small Value )
( 6 $ 4 = 18 Large Height )
( 8 $ 5 = 30 Large Value )
)
(
( 8 $ 5 = 30 Large Height )
( 5 $ 3 = 12 Small Value )
( 9 $ 4 = 30 Large Base )
( 5 $ 5 = 15 Large Height )
( 8 $ 4 = 26 Large Value )
( 9 $ 3 = 24 Small Base )
( 7 $ 3 = 18 Small Height )
( 4 $ 5 = 10 Large Value )
( 4 $ 2 = 7 Small Value )
( 4 $ 5 = 10 Large Base )
( 6 $ 3 = 15 Small Height )
)
))
( 10 (
(
( 9 $ 4 = 30 Large Value )
( 7 $ 2 = 13 Small Base )
( 7 $ 3 = 18 Small Base )
( 9 $ 2 = 17 Small Value )
( 9 $ 4 = 30 Large Base )
( 8 $ 5 = 30 Large Height )
( 7 $ 5 = 25 Large Height )
( 7 $ 3 = 18 Small Height )
( 5 $ 3 = 12 Small Value )
)
(
( 4 $ 3 = 9 Small Base )
( 9 $ 5 = 35 Large Base )
( 4 $ 4 = 10 Large Value )
( 6 $ 3 = 15 Small Height )
( 6 $ 4 = 18 Large Height )
( 4 $ 5 = 10 Large Base )
( 9 $ 2 = 17 Small Base )
( 4 $ 2 = 7 Small Value )
( 5 $ 4 = 14 Large Value )
( 4 $ 2 = 7 Small Height )
( 8 $ 4 = 26 Large Height )
)
(
( 5 $ 3 = 12 Small Base )
( 7 $ 2 = 13 Small Height )
( 7 $ 3 = 18 Small Value )
( 8 $ 2 = 15 Small Value )
( 4 $ 5 = 10 Large Height )
( 6 $ 2 = 11 Small Height )
( 4 $ 5 = 10 Large Value )
( 6 $ 5 = 20 Large Base )
( 8 $ 4 = 26 Large Base )
( 8 $ 5 = 30 Large Value )
( 6 $ 3 = 15 Small Base )
( 7 $ 4 = 22 Large Height )
)
(
( 8 $ 3 = 21 Small Height )
( 6 $ 4 = 18 Large Value )
( 5 $ 4 = 14 Large Base )
( 8 $ 3 = 21 Small Base )
( 6 $ 2 = 11 Small Value )
( 8 $ 4 = 26 Large Value )
( 5 $ 4 = 14 Large Height )
( 9 $ 2 = 17 Small Height )
( 6 $ 5 = 20 Large Height )
( 9 $ 3 = 24 Small Base )
( 5 $ 5 = 15 Large Base )
( 6 $ 3 = 15 Small Value )
)
(
( 4 $ 4 = 10 Large Height )
( 5 $ 2 = 9 Small Base )
( 6 $ 2 = 11 Small Base )
( 5 $ 5 = 15 Large Height )
( 5 $ 3 = 12 Small Height )
( 7 $ 5 = 25 Large Base )
( 7 $ 2 = 13 Small Value )
( 5 $ 5 = 15 Large Value )
( 8 $ 2 = 15 Small Height )
( 4 $ 3 = 9 Small Value )
( 4 $ 4 = 10 Large Base )
)
(
( 7 $ 4 = 22 Large Value )
( 6 $ 4 = 18 Large Base )
( 8 $ 2 = 15 Small Base )
( 5 $ 2 = 9 Small Height )
( 4 $ 2 = 7 Small Base )
( 5 $ 2 = 9 Small Value )
( 7 $ 4 = 22 Large Base )
( 8 $ 3 = 21 Small Value )
( 7 $ 5 = 25 Large Value )
( 4 $ 3 = 9 Small Height )
)
))
( 11 (
(
( 8 $ 2 = 15 Small Base )
( 7 $ 2 = 13 Small Value )
( 8 $ 2 = 15 Small Height )
( 6 $ 2 = 11 Small Height )
( 8 $ 3 = 21 Small Base )
( 6 $ 5 = 20 Large Base )
( 6 $ 5 = 20 Large Value )
( 8 $ 5 = 30 Large Height )
( 8 $ 2 = 15 Small Value )
( 5 $ 5 = 15 Large Value )
( 9 $ 4 = 30 Large Base )
)
(
( 8 $ 3 = 21 Small Height )
( 8 $ 4 = 26 Large Height )
( 9 $ 5 = 35 Large Height )
( 4 $ 5 = 10 Large Base )
( 4 $ 3 = 9 Small Base )
( 8 $ 4 = 26 Large Base )
( 4 $ 3 = 9 Small Height )
( 8 $ 5 = 30 Large Value )
( 5 $ 4 = 14 Large Value )
( 4 $ 3 = 9 Small Value )
( 8 $ 3 = 21 Small Value )
)
(
( 6 $ 4 = 18 Large Value )
( 4 $ 4 = 10 Large Height )
( 6 $ 3 = 15 Small Height )
( 4 $ 2 = 7 Small Base )
( 4 $ 2 = 7 Small Value )
( 7 $ 3 = 18 Small Height )
( 7 $ 4 = 22 Large Height )
( 7 $ 2 = 13 Small Base )
( 5 $ 5 = 15 Large Base )
)
(
( 5 $ 3 = 12 Small Value )
( 9 $ 2 = 17 Small Height )
( 6 $ 4 = 18 Large Height )
( 6 $ 3 = 15 Small Value )
( 5 $ 3 = 12 Small Height )
( 9 $ 2 = 17 Small Base )
( 4 $ 4 = 10 Large Base )
( 7 $ 5 = 25 Large Value )
( 7 $ 4 = 22 Large Base )
( 4 $ 4 = 10 Large Value )
)
(
( 8 $ 4 = 26 Large Value )
( 7 $ 5 = 25 Large Height )
( 5 $ 4 = 14 Large Height )
( 5 $ 2 = 9 Small Height )
( 4 $ 2 = 7 Small Height )
( 9 $ 3 = 24 Small Base )
( 6 $ 4 = 18 Large Base )
( 5 $ 2 = 9 Small Base )
( 9 $ 5 = 35 Large Value )
( 5 $ 2 = 9 Small Value )
)
(
( 6 $ 2 = 11 Small Base )
( 9 $ 3 = 24 Small Height )
( 6 $ 2 = 11 Small Value )
( 4 $ 5 = 10 Large Height )
( 5 $ 3 = 12 Small Base )
( 7 $ 2 = 13 Small Height )
( 5 $ 5 = 15 Large Height )
( 7 $ 4 = 22 Large Value )
( 5 $ 4 = 14 Large Base )
( 9 $ 2 = 17 Small Value )
( 4 $ 5 = 10 Large Value )
( 9 $ 5 = 35 Large Base )
)
))
( 12 (
(
( 6 $ 4 = 18 Large Height )
( 6 $ 3 = 15 Small Value )
( 6 $ 2 = 11 Small Base )
( 5 $ 2 = 9 Small Height )
( 8 $ 4 = 26 Large Base )
( 6 $ 5 = 20 Large Value )
( 9 $ 3 = 24 Small Value )
( 4 $ 3 = 9 Small Base )
( 6 $ 2 = 11 Small Height )
)
(
( 7 $ 4 = 22 Large Height )
( 8 $ 5 = 30 Large Value )
( 9 $ 5 = 35 Large Base )
( 5 $ 2 = 9 Small Value )
( 8 $ 2 = 15 Small Height )
( 6 $ 3 = 15 Small Base )
( 9 $ 2 = 17 Small Base )
( 8 $ 3 = 21 Small Height )
( 4 $ 4 = 10 Large Value )
( 4 $ 3 = 9 Small Value )
( 4 $ 5 = 10 Large Base )
)
(
( 6 $ 2 = 11 Small Value )
( 4 $ 3 = 9 Small Height )
( 8 $ 5 = 30 Large Height )
( 5 $ 4 = 14 Large Value )
( 5 $ 3 = 12 Small Value )
( 5 $ 4 = 14 Large Base )
( 4 $ 5 = 10 Large Height )
( 9 $ 2 = 17 Small Height )
)
(
( 9 $ 5 = 35 Large Value )
( 9 $ 4 = 30 Large Height )
( 7 $ 3 = 18 Small Base )
( 9 $ 4 = 30 Large Value )
( 4 $ 2 = 7 Small Base )
( 6 $ 5 = 20 Large Base )
( 4 $ 2 = 7 Small Value )
( 9 $ 2 = 17 Small Value )
( 6 $ 3 = 15 Small Height )
( 8 $ 5 = 30 Large Base )
( 7 $ 2 = 13 Small Height )
)
(
( 5 $ 4 = 14 Large Height )
( 7 $ 2 = 13 Small Value )
( 9 $ 4 = 30 Large Base )
( 9 $ 5 = 35 Large Height )
( 6 $ 4 = 18 Large Value )
( 8 $ 2 = 15 Small Value )
( 4 $ 4 = 10 Large Base )
( 5 $ 3 = 12 Small Height )
( 5 $ 2 = 9 Small Base )
)
(
( 6 $ 4 = 18 Large Base )
( 4 $ 5 = 10 Large Value )
( 8 $ 3 = 21 Small Base )
( 7 $ 3 = 18 Small Value )
( 7 $ 3 = 18 Small Height )
( 5 $ 5 = 15 Large Height )
( 7 $ 5 = 25 Large Height )
( 8 $ 3 = 21 Small Value )
( 4 $ 2 = 7 Small Height )
( 5 $ 5 = 15 Large Value )
)
))
( 13 (
(
( 6 $ 2 = 11 Small Base )
( 6 $ 2 = 11 Small Value )
( 9 $ 4 = 30 Large Base )
( 5 $ 5 = 15 Large Value )
( 9 $ 2 = 17 Small Height )
( 9 $ 2 = 17 Small Value )
( 8 $ 4 = 26 Large Height )
( 7 $ 4 = 22 Large Base )
)
(
( 7 $ 2 = 13 Small Base )
( 4 $ 5 = 10 Large Height )
( 8 $ 2 = 15 Small Height )
( 4 $ 3 = 9 Small Base )
( 6 $ 4 = 18 Large Base )
( 5 $ 4 = 14 Large Value )
( 7 $ 3 = 18 Small Value )
( 8 $ 2 = 15 Small Value )
( 5 $ 5 = 15 Large Height )
)
(
( 5 $ 5 = 15 Large Base )
( 5 $ 3 = 12 Small Height )
( 4 $ 5 = 10 Large Value )
( 4 $ 4 = 10 Large Value )
( 4 $ 2 = 7 Small Base )
( 8 $ 3 = 21 Small Height )
( 9 $ 4 = 30 Large Height )
( 8 $ 3 = 21 Small Base )
( 8 $ 4 = 26 Large Base )
( 4 $ 3 = 9 Small Value )
( 8 $ 3 = 21 Small Value )
( 6 $ 5 = 20 Large Height )
)
(
( 6 $ 4 = 18 Large Value )
( 5 $ 3 = 12 Small Value )
( 4 $ 2 = 7 Small Height )
( 4 $ 5 = 10 Large Base )
( 7 $ 4 = 22 Large Height )
( 5 $ 3 = 12 Small Base )
( 9 $ 5 = 35 Large Base )
( 5 $ 4 = 14 Large Height )
( 4 $ 2 = 7 Small Value )
( 9 $ 2 = 17 Small Base )
( 7 $ 3 = 18 Small Height )
)
(
( 7 $ 5 = 25 Large Base )
( 9 $ 5 = 35 Large Value )
( 7 $ 5 = 25 Large Value )
( 7 $ 2 = 13 Small Value )
( 4 $ 4 = 10 Large Height )
( 6 $ 3 = 15 Small Base )
( 5 $ 2 = 9 Small Value )
( 6 $ 4 = 18 Large Height )
( 8 $ 5 = 30 Large Base )
( 7 $ 2 = 13 Small Height )
( 6 $ 2 = 11 Small Height )
( 8 $ 2 = 15 Small Base )
)
(
( 9 $ 5 = 35 Large Height )
( 5 $ 2 = 9 Small Base )
( 9 $ 4 = 30 Large Value )
( 7 $ 4 = 22 Large Value )
( 4 $ 4 = 10 Large Base )
( 7 $ 5 = 25 Large Height )
( 7 $ 3 = 18 Small Base )
( 5 $ 2 = 9 Small Height )
( 5 $ 4 = 14 Large Base )
( 6 $ 3 = 15 Small Height )
( 6 $ 3 = 15 Small Value )
)
))
( 14 (
(
( 4 $ 2 = 7 Small Height )
( 4 $ 3 = 9 Small Value )
( 5 $ 5 = 15 Large Value )
( 9 $ 5 = 35 Large Height )
( 4 $ 4 = 10 Large Height )
( 7 $ 3 = 18 Small Base )
( 7 $ 2 = 13 Small Value )
( 5 $ 3 = 12 Small Height )
( 9 $ 4 = 30 Large Value )
( 7 $ 4 = 22 Large Base )
( 4 $ 3 = 9 Small Base )
( 4 $ 4 = 10 Large Base )
)
(
( 9 $ 2 = 17 Small Height )
( 8 $ 4 = 26 Large Base )
( 5 $ 3 = 12 Small Base )
( 8 $ 3 = 21 Small Value )
( 5 $ 3 = 12 Small Value )
( 6 $ 4 = 18 Large Value )
( 5 $ 2 = 9 Small Base )
( 7 $ 2 = 13 Small Height )
( 7 $ 4 = 22 Large Height )
( 7 $ 5 = 25 Large Height )
( 8 $ 4 = 26 Large Value )
( 6 $ 5 = 20 Large Base )
)
(
( 4 $ 5 = 10 Large Value )
( 9 $ 3 = 24 Small Base )
( 6 $ 5 = 20 Large Height )
( 6 $ 2 = 11 Small Height )
( 8 $ 2 = 15 Small Base )
( 5 $ 5 = 15 Large Base )
( 8 $ 4 = 26 Large Height )
( 7 $ 3 = 18 Small Value )
( 9 $ 4 = 30 Large Base )
( 6 $ 5 = 20 Large Value )
( 5 $ 2 = 9 Small Height )
( 8 $ 2 = 15 Small Value )
)
(
( 8 $ 3 = 21 Small Height )
( 8 $ 5 = 30 Large Value )
( 7 $ 2 = 13 Small Base )
( 5 $ 2 = 9 Small Value )
( 6 $ 2 = 11 Small Value )
( 7 $ 4 = 22 Large Value )
( 8 $ 3 = 21 Small Base )
( 9 $ 5 = 35 Large Base )
( 8 $ 5 = 30 Large Height )
( 8 $ 2 = 15 Small Height )
)
(
( 4 $ 5 = 10 Large Height )
( 6 $ 3 = 15 Small Height )
( 4 $ 4 = 10 Large Value )
( 6 $ 3 = 15 Small Base )
( 8 $ 5 = 30 Large Base )
( 4 $ 5 = 10 Large Base )
( 6 $ 3 = 15 Small Value )
( 5 $ 5 = 15 Large Height )
( 9 $ 2 = 17 Small Base )
( 9 $ 3 = 24 Small Height )
)
(
( 7 $ 3 = 18 Small Height )
( 4 $ 2 = 7 Small Base )
( 5 $ 4 = 14 Large Value )
( 4 $ 2 = 7 Small Value )
( 9 $ 3 = 24 Small Value )
( 7 $ 5 = 25 Large Value )
( 6 $ 4 = 18 Large Base )
( 7 $ 5 = 25 Large Base )
( 4 $ 3 = 9 Small Height )
)
))
( 15 (
(
( 7 $ 4 = 22 Large Height )
( 4 $ 5 = 10 Large Height )
( 5 $ 2 = 9 Small Height )
( 8 $ 4 = 26 Large Value )
( 9 $ 2 = 17 Small Value )
( 4 $ 3 = 9 Small Height )
( 6 $ 2 = 11 Small Value )
( 8 $ 2 = 15 Small Base )
( 7 $ 5 = 25 Large Value )
( 4 $ 4 = 10 Large Base )
( 7 $ 2 = 13 Small Base )
)
(
( 7 $ 3 = 18 Small Height )
( 8 $ 3 = 21 Small Height )
( 5 $ 5 = 15 Large Height )
( 5 $ 3 = 12 Small Base )
( 8 $ 2 = 15 Small Value )
( 6 $ 4 = 18 Large Base )
( 7 $ 3 = 18 Small Base )
( 8 $ 5 = 30 Large Height )
( 9 $ 4 = 30 Large Value )
( 6 $ 5 = 20 Large Value )
( 5 $ 2 = 9 Small Value )
)
(
( 7 $ 5 = 25 Large Height )
( 9 $ 4 = 30 Large Height )
( 7 $ 2 = 13 Small Value )
( 9 $ 2 = 17 Small Base )
( 8 $ 5 = 30 Large Value )
( 4 $ 2 = 7 Small Base )
( 9 $ 5 = 35 Large Base )
( 7 $ 4 = 22 Large Base )
( 7 $ 3 = 18 Small Value )
( 9 $ 5 = 35 Large Value )
)
(
( 4 $ 2 = 7 Small Height )
( 5 $ 4 = 14 Large Height )
( 6 $ 3 = 15 Small Base )
( 5 $ 2 = 9 Small Base )
( 8 $ 4 = 26 Large Height )
( 8 $ 5 = 30 Large Base )
( 4 $ 3 = 9 Small Value )
( 4 $ 4 = 10 Large Value )
( 8 $ 4 = 26 Large Base )
( 5 $ 3 = 12 Small Value )
( 6 $ 3 = 15 Small Height )
)
(
( 8 $ 3 = 21 Small Base )
( 8 $ 3 = 21 Small Value )
( 5 $ 5 = 15 Large Value )
( 9 $ 5 = 35 Large Height )
( 5 $ 4 = 14 Large Value )
( 7 $ 5 = 25 Large Base )
( 4 $ 5 = 10 Large Base )
( 4 $ 2 = 7 Small Value )
( 6 $ 2 = 11 Small Height )
( 9 $ 3 = 24 Small Height )
( 6 $ 2 = 11 Small Base )
( 6 $ 4 = 18 Large Height )
)
(
( 9 $ 2 = 17 Small Height )
( 6 $ 3 = 15 Small Value )
( 7 $ 2 = 13 Small Height )
( 4 $ 4 = 10 Large Height )
( 6 $ 4 = 18 Large Value )
( 7 $ 4 = 22 Large Value )
( 6 $ 5 = 20 Large Height )
( 9 $ 3 = 24 Small Base )
( 9 $ 3 = 24 Small Value )
( 4 $ 3 = 9 Small Base )
( 5 $ 5 = 15 Large Base )
( 6 $ 5 = 20 Large Base )
)
))
( 16 (
(
( 4 $ 5 = 10 Large Value )
( 8 $ 2 = 15 Small Height )
( 5 $ 3 = 12 Small Value )
( 5 $ 4 = 14 Large Base )
( 8 $ 4 = 26 Large Height )
( 6 $ 3 = 15 Small Height )
( 6 $ 5 = 20 Large Height )
( 4 $ 4 = 10 Large Value )
( 5 $ 2 = 9 Small Base )
( 9 $ 2 = 17 Small Value )
( 4 $ 3 = 9 Small Base )
)
(
( 9 $ 3 = 24 Small Value )
( 5 $ 3 = 12 Small Base )
( 9 $ 5 = 35 Large Height )
( 7 $ 4 = 22 Large Base )
( 8 $ 5 = 30 Large Value )
( 8 $ 4 = 26 Large Value )
( 8 $ 3 = 21 Small Value )
( 6 $ 2 = 11 Small Base )
( 8 $ 5 = 30 Large Height )
( 8 $ 3 = 21 Small Height )
)
(
( 4 $ 5 = 10 Large Height )
( 4 $ 3 = 9 Small Value )
( 7 $ 2 = 13 Small Value )
( 5 $ 5 = 15 Large Value )
( 6 $ 3 = 15 Small Base )
( 7 $ 3 = 18 Small Height )
( 7 $ 5 = 25 Large Value )
( 9 $ 4 = 30 Large Height )
( 7 $ 3 = 18 Small Base )
( 9 $ 4 = 30 Large Base )
( 7 $ 2 = 13 Small Height )
)
(
( 8 $ 2 = 15 Small Base )
( 9 $ 2 = 17 Small Height )
( 4 $ 4 = 10 Large Height )
( 8 $ 5 = 30 Large Base )
( 8 $ 3 = 21 Small Base )
( 6 $ 4 = 18 Large Value )
( 6 $ 2 = 11 Small Value )
( 4 $ 4 = 10 Large Base )
( 7 $ 4 = 22 Large Value )
( 9 $ 3 = 24 Small Height )
( 4 $ 2 = 7 Small Value )
( 6 $ 4 = 18 Large Height )
)
(
( 4 $ 5 = 10 Large Base )
( 5 $ 5 = 15 Large Height )
( 5 $ 4 = 14 Large Height )
( 6 $ 5 = 20 Large Value )
( 5 $ 2 = 9 Small Value )
( 7 $ 2 = 13 Small Base )
( 7 $ 3 = 18 Small Value )
( 4 $ 2 = 7 Small Height )
( 6 $ 2 = 11 Small Height )
( 4 $ 2 = 7 Small Base )
( 9 $ 5 = 35 Large Value )
)
(
( 6 $ 3 = 15 Small Value )
( 9 $ 3 = 24 Small Base )
( 9 $ 4 = 30 Large Value )
( 7 $ 4 = 22 Large Height )
( 5 $ 4 = 14 Large Value )
( 9 $ 2 = 17 Small Base )
( 7 $ 5 = 25 Large Height )
( 5 $ 2 = 9 Small Height )
( 5 $ 5 = 15 Large Base )
( 8 $ 2 = 15 Small Value )
)
))
( 17 (
(
( 9 $ 4 = 30 Large Height )
( 9 $ 4 = 30 Large Value )
( 8 $ 3 = 21 Small Height )
( 9 $ 3 = 24 Small Value )
( 6 $ 5 = 20 Large Base )
( 7 $ 3 = 18 Small Value )
( 9 $ 3 = 24 Small Base )
( 6 $ 3 = 15 Small Base )
( 4 $ 4 = 10 Large Base )
( 8 $ 2 = 15 Small Height )
( 8 $ 5 = 30 Large Height )
)
(
( 8 $ 5 = 30 Large Base )
( 6 $ 2 = 11 Small Base )
( 6 $ 4 = 18 Large Value )
( 9 $ 5 = 35 Large Height )
( 9 $ 2 = 17 Small Value )
( 8 $ 4 = 26 Large Height )
( 8 $ 4 = 26 Large Value )
( 5 $ 4 = 14 Large Base )
( 4 $ 2 = 7 Small Base )
( 7 $ 3 = 18 Small Height )
( 4 $ 3 = 9 Small Height )
( 4 $ 3 = 9 Small Value )
)
(
( 7 $ 2 = 13 Small Base )
( 6 $ 3 = 15 Small Height )
( 8 $ 3 = 21 Small Value )
( 9 $ 5 = 35 Large Value )
( 6 $ 2 = 11 Small Height )
( 5 $ 5 = 15 Large Base )
( 5 $ 3 = 12 Small Base )
( 4 $ 2 = 7 Small Value )
( 6 $ 5 = 20 Large Value )
( 6 $ 4 = 18 Large Height )
( 4 $ 5 = 10 Large Height )
)
(
( 8 $ 2 = 15 Small Value )
( 9 $ 3 = 24 Small Height )
( 7 $ 4 = 22 Large Height )
( 4 $ 4 = 10 Large Value )
( 7 $ 5 = 25 Large Height )
( 6 $ 4 = 18 Large Base )
( 6 $ 3 = 15 Small Value )
( 5 $ 3 = 12 Small Height )
( 5 $ 2 = 9 Small Base )
( 7 $ 3 = 18 Small Base )
( 7 $ 4 = 22 Large Value )
( 7 $ 4 = 22 Large Base )
)
(
( 5 $ 5 = 15 Large Value )
( 4 $ 5 = 10 Large Value )
( 5 $ 2 = 9 Small Height )
( 7 $ 2 = 13 Small Height )
( 6 $ 2 = 11 Small Value )
( 9 $ 2 = 17 Small Base )
( 5 $ 4 = 14 Large Height )
( 9 $ 4 = 30 Large Base )
( 4 $ 3 = 9 Small Base )
( 4 $ 4 = 10 Large Height )
( 5 $ 2 = 9 Small Value )
( 9 $ 5 = 35 Large Base )
)
(
( 8 $ 3 = 21 Small Base )
( 7 $ 5 = 25 Large Base )
( 7 $ 2 = 13 Small Value )
( 6 $ 5 = 20 Large Height )
( 8 $ 2 = 15 Small Base )
( 8 $ 5 = 30 Large Value )
( 7 $ 5 = 25 Large Value )
( 5 $ 5 = 15 Large Height )
( 4 $ 5 = 10 Large Base )
( 9 $ 2 = 17 Small Height )
( 4 $ 2 = 7 Small Height )
( 5 $ 3 = 12 Small Value )
)
))
( 18 (
(
( 4 $ 4 = 10 Large Base )
( 8 $ 5 = 30 Large Height )
( 7 $ 4 = 22 Large Value )
( 9 $ 2 = 17 Small Height )
( 8 $ 2 = 15 Small Height )
( 6 $ 4 = 18 Large Value )
( 8 $ 5 = 30 Large Base )
( 5 $ 2 = 9 Small Base )
( 5 $ 3 = 12 Small Value )
( 7 $ 2 = 13 Small Base )
( 6 $ 3 = 15 Small Value )
( 5 $ 5 = 15 Large Height )
)
(
( 9 $ 4 = 30 Large Height )
( 4 $ 3 = 9 Small Height )
( 5 $ 5 = 15 Large Base )
( 6 $ 5 = 20 Large Value )
( 9 $ 5 = 35 Large Value )
( 8 $ 3 = 21 Small Height )
( 4 $ 3 = 9 Small Value )
( 7 $ 3 = 18 Small Base )
( 9 $ 5 = 35 Large Base )
( 5 $ 2 = 9 Small Value )
( 9 $ 2 = 17 Small Base )
( 5 $ 4 = 14 Large Height )
)
(
( 6 $ 5 = 20 Large Height )
( 7 $ 2 = 13 Small Value )
( 7 $ 3 = 18 Small Height )
( 6 $ 3 = 15 Small Base )
( 6 $ 2 = 11 Small Value )
( 4 $ 5 = 10 Large Value )
( 8 $ 3 = 21 Small Base )
( 7 $ 5 = 25 Large Value )
)
(
( 4 $ 2 = 7 Small Base )
( 5 $ 5 = 15 Large Value )
( 5 $ 3 = 12 Small Base )
( 5 $ 4 = 14 Large Value )
( 8 $ 3 = 21 Small Value )
( 4 $ 5 = 10 Large Height )
( 6 $ 3 = 15 Small Height )
( 6 $ 2 = 11 Small Height )
( 8 $ 2 = 15 Small Value )
( 7 $ 5 = 25 Large Base )
( 5 $ 4 = 14 Large Base )
( 8 $ 4 = 26 Large Height )
)
(
( 9 $ 4 = 30 Large Value )
( 9 $ 3 = 24 Small Base )
( 4 $ 2 = 7 Small Value )
( 4 $ 2 = 7 Small Height )
( 5 $ 3 = 12 Small Height )
( 4 $ 4 = 10 Large Value )
( 9 $ 5 = 35 Large Height )
( 6 $ 4 = 18 Large Base )
( 6 $ 2 = 11 Small Base )
( 4 $ 4 = 10 Large Height )
( 7 $ 3 = 18 Small Value )
)
(
( 8 $ 5 = 30 Large Value )
( 7 $ 4 = 22 Large Base )
( 9 $ 3 = 24 Small Value )
( 8 $ 2 = 15 Small Base )
( 9 $ 2 = 17 Small Value )
( 7 $ 4 = 22 Large Height )
( 4 $ 3 = 9 Small Base )
( 6 $ 4 = 18 Large Height )
( 9 $ 3 = 24 Small Height )
( 8 $ 4 = 26 Large Value )
( 9 $ 4 = 30 Large Base )
( 5 $ 2 = 9 Small Height )
)
))
))
